"""
Snowflake Role API.

The Snowflake Role API is a REST API that you can use to access, update, and perform certain actions on Role resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictBool, StrictStr

from snowflake.core.role._generated.models.containing_scope import ContainingScope, ContainingScopeModel
from snowflake.core.role._generated.models.securable import Securable, SecurableModel


class Grant(BaseModel):
    """A model object representing the Grant resource.

    Constructs an object of type Grant with the provided properties.

    Parameters
    __________
    securable_type : str
        Type of the securable to be granted.
    securable : Securable, optional

    containing_scope : ContainingScope, optional

    grant_option : bool, optional
        If true, allows the recipient role to grant the privileges to other roles.
    privileges : list[str], optional
        List of privileges to be granted.
    created_on : datetime, optional
        Date and time when the grant was created — **Read-only:** *any user-provided value will be ignored.*
    granted_by : str, optional
        The role that granted this privilege to this grantee — **Read-only:** *any user-provided value will be ignored.*
    """

    securable: Optional[Securable] = None

    containing_scope: Optional[ContainingScope] = None

    securable_type: StrictStr

    grant_option: Optional[StrictBool] = None

    privileges: Optional[List[StrictStr]] = None

    created_on: Optional[datetime] = None

    granted_by: Optional[StrictStr] = None

    __properties = [
        "securable",
        "containing_scope",
        "securable_type",
        "grant_option",
        "privileges",
        "created_on",
        "granted_by",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Grant:
        """Create an instance of Grant from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "granted_by",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of securable
        if self.securable:
            _dict["securable"] = self.securable.to_dict()

        # override the default output from pydantic by calling `to_dict()` of containing_scope
        if self.containing_scope:
            _dict["containing_scope"] = self.containing_scope.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Grant:
        """Create an instance of Grant from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Grant.model_validate(obj)

        _obj = Grant.model_validate(
            {
                "securable": Securable.from_dict(obj.get("securable")) if obj.get("securable") is not None else None,
                "containing_scope": ContainingScope.from_dict(obj.get("containing_scope"))
                if obj.get("containing_scope") is not None
                else None,
                "securable_type": obj.get("securable_type"),
                "grant_option": obj.get("grant_option"),
                "privileges": obj.get("privileges"),
                "created_on": obj.get("created_on"),
                "granted_by": obj.get("granted_by"),
            }
        )

        return _obj


class GrantModel:
    def __init__(
        self,
        securable_type: str,
        # optional properties
        securable: Optional[Securable] = None,
        containing_scope: Optional[ContainingScope] = None,
        grant_option: Optional[bool] = None,
        privileges: Optional[list[str]] = None,
        created_on: Optional[datetime] = None,
        granted_by: Optional[str] = None,
    ):
        """A model object representing the Grant resource.

        Constructs an object of type Grant with the provided properties.

        Parameters
        __________
        securable_type : str
            Type of the securable to be granted.
        securable : Securable, optional

        containing_scope : ContainingScope, optional

        grant_option : bool, optional
            If true, allows the recipient role to grant the privileges to other roles.
        privileges : list[str], optional
            List of privileges to be granted.
        created_on : datetime, optional
            Date and time when the grant was created
        granted_by : str, optional
            The role that granted this privilege to this grantee
        """
        self.securable = securable
        self.containing_scope = containing_scope
        self.securable_type = securable_type
        self.grant_option = grant_option
        self.privileges = privileges
        self.created_on = created_on
        self.granted_by = granted_by

    __properties = [
        "securable",
        "containing_scope",
        "securable_type",
        "grant_option",
        "privileges",
        "created_on",
        "granted_by",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Grant(
            securable=self.securable._to_model() if self.securable is not None else None,
            containing_scope=self.containing_scope._to_model() if self.containing_scope is not None else None,
            securable_type=self.securable_type,
            grant_option=self.grant_option,
            privileges=self.privileges,
            created_on=self.created_on,
            granted_by=self.granted_by,
        )

    @classmethod
    def _from_model(cls, model) -> GrantModel:
        return GrantModel(
            securable=SecurableModel._from_model(model.securable) if model.securable else None,
            containing_scope=ContainingScopeModel._from_model(model.containing_scope)
            if model.containing_scope
            else None,
            securable_type=model.securable_type,
            grant_option=model.grant_option,
            privileges=model.privileges,
            created_on=model.created_on,
            granted_by=model.granted_by,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Grant.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Grant object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> GrantModel:
        """Create an instance of Grant from a dict.

        This method constructs a Grant object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Grant
            A Grant object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Grant.from_dict(obj))


Grant._model_class = GrantModel



from contextlib import suppress

import pytest

from snowflake.core._common import CreateMode
from snowflake.core.catalog_integration import CatalogIntegration, Glue, OAuth, ObjectStore, Polaris, RestConfig
from snowflake.core.exceptions import ConflictError, NotFoundError
from tests.integ.utils import random_string


pytestmark = [
    pytest.mark.min_sf_ver("8.37.0"),
    pytest.mark.internal_only,
    pytest.mark.usefixtures("setup_credentials_fixture"),
]


def test_create_by_glue_and_fetch(catalog_integrations):
    catalog_integration_name = random_string(10, "test_catalog_integration_")

    catalog_integration = CatalogIntegration(
        name=catalog_integration_name,
        catalog=Glue(
            catalog_namespace="abcd-ns",
            glue_aws_role_arn="arn:aws:iam::123456789012:role/sqsAccess",
            glue_catalog_id="1234567",
        ),
        table_format="ICEBERG",
        enabled=True,
    )

    catalog_integration_handle = catalog_integrations.create(catalog_integration)

    try:
        fetch_handle = catalog_integration_handle.fetch()

        assert fetch_handle.name.upper() == catalog_integration_name.upper()
        assert isinstance(fetch_handle.catalog, Glue)
        assert fetch_handle.catalog.catalog_namespace == catalog_integration.catalog.catalog_namespace
        assert fetch_handle.catalog.glue_aws_role_arn == catalog_integration.catalog.glue_aws_role_arn
        assert fetch_handle.catalog.glue_catalog_id == catalog_integration.catalog.glue_catalog_id
        assert fetch_handle.table_format == catalog_integration.table_format
        assert fetch_handle.enabled == catalog_integration.enabled

    finally:
        with suppress(NotFoundError):
            catalog_integration_handle.drop()


def test_create_by_object_store_and_fetch(catalog_integrations):
    catalog_integration_name = random_string(10, "test_catalog_integration_")

    catalog_integration = CatalogIntegration(
        name=catalog_integration_name, catalog=ObjectStore(), table_format="ICEBERG", enabled=True
    )

    catalog_integration_handle = catalog_integrations.create(catalog_integration)

    try:
        fetch_handle = catalog_integration_handle.fetch()

        assert fetch_handle.name.upper() == catalog_integration_name.upper()
        assert isinstance(fetch_handle.catalog, ObjectStore)
        assert fetch_handle.table_format == catalog_integration.table_format
        assert fetch_handle.enabled == catalog_integration.enabled
    finally:
        with suppress(NotFoundError):
            catalog_integration_handle.drop()


def test_create_by_polaris_and_fetch(catalog_integrations, set_internal_params):
    catalog_integration_name = random_string(10, "test_catalog_integration_")

    with set_internal_params({"ENABLE_FIX_1692844_ENABLE_VALIDATION_ON_CATALOG_INT_CREATION": False}):
        catalog_integration = CatalogIntegration(
            name=catalog_integration_name,
            catalog=Polaris(
                catalog_namespace="abcd-ns",
                rest_config=RestConfig(
                    catalog_uri="https://my_account.snowflakecomputing.com/polaris/api/catalog",
                    warehouse="my-warehouse",
                ),
                rest_authentication=OAuth(
                    type="OAUTH",
                    oauth_client_id="my_client_id",
                    oauth_client_secret="my_client_secret",
                    oauth_allowed_scopes=["PRINCIPAL_ROLE:ALL"],
                ),
            ),
            table_format="ICEBERG",
            enabled=True,
        )

        catalog_integration_handle = catalog_integrations.create(catalog_integration)

        try:
            fetch_handle = catalog_integration_handle.fetch()

            assert fetch_handle.name.upper() == catalog_integration_name.upper()
            assert isinstance(fetch_handle.catalog, Polaris)
            assert fetch_handle.catalog.catalog_namespace == catalog_integration.catalog.catalog_namespace
            assert fetch_handle.catalog.rest_config.catalog_uri == catalog_integration.catalog.rest_config.catalog_uri
            assert fetch_handle.catalog.rest_config.warehouse == catalog_integration.catalog.rest_config.warehouse
            assert (
                fetch_handle.catalog.rest_authentication.oauth_client_id
                == catalog_integration.catalog.rest_authentication.oauth_client_id
            )
            assert fetch_handle.catalog.rest_authentication.oauth_client_secret == ""
            assert (
                fetch_handle.catalog.rest_authentication.oauth_allowed_scopes
                == catalog_integration.catalog.rest_authentication.oauth_allowed_scopes
            )
            assert fetch_handle.table_format == catalog_integration.table_format
            assert fetch_handle.enabled == catalog_integration.enabled
        finally:
            with suppress(NotFoundError):
                catalog_integration_handle.drop()


def test_create_mode(catalog_integrations):
    catalog_integration_name = random_string(10, "test_catalog_integration_")

    catalog_integration = CatalogIntegration(
        name=catalog_integration_name, catalog=ObjectStore(), table_format="ICEBERG", enabled=True
    )

    catalog_integrations.create(catalog_integration)

    with pytest.raises(ConflictError):
        catalog_integrations.create(catalog_integration)

    catalog_integrations.create(catalog_integration, mode=CreateMode.or_replace)



"""
Cortex Chat API.

OpenAPI 3.0 specification for the Cortex Chat API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict, StrictBool, StrictInt, StrictStr

from snowflake.core.cortex.chat_service._generated.models.chat_request_messages_inner import (
    ChatRequestMessagesInner,
    ChatRequestMessagesInnerModel,
)
from snowflake.core.cortex.chat_service._generated.models.chat_request_search_services_inner import (
    ChatRequestSearchServicesInner,
    ChatRequestSearchServicesInnerModel,
)
from snowflake.core.cortex.chat_service._generated.models.chat_request_semantic_models_inner import (
    ChatRequestSemanticModelsInner,
    ChatRequestSemanticModelsInnerModel,
)


class ChatRequest(BaseModel):
    """A model object representing the ChatRequest resource.

    Constructs an object of type ChatRequest with the provided properties.

    Parameters
    __________
    query : str
        Unstructured text query.
    model : str, optional
        The model to be used for the chat.
    search_services : list[ChatRequestSearchServicesInner], optional
        List of search services.
    semantic_models : list[ChatRequestSemanticModelsInner], optional
        The semantic models to be used for Cortex Analyst
    prompt : str, optional
        Additional prompt text.
    messages : list[ChatRequestMessagesInner], optional
        List of conversation messages.
    debug : bool, optional
        Whether to return debug information.
    max_results : int, optional
        Global max number of results from search for llm generation. If not set, default to sum of max_results across list of search_service.
    experimental : object, optional
        reserved
    """

    query: StrictStr

    model: Optional[StrictStr] = None

    search_services: Optional[List[ChatRequestSearchServicesInner]] = None

    semantic_models: Optional[List[ChatRequestSemanticModelsInner]] = None

    prompt: Optional[StrictStr] = None

    messages: Optional[List[ChatRequestMessagesInner]] = None

    debug: Optional[StrictBool] = None

    max_results: Optional[StrictInt] = None

    experimental: Optional[Dict[str, Any]] = None

    __properties = [
        "query",
        "model",
        "search_services",
        "semantic_models",
        "prompt",
        "messages",
        "debug",
        "max_results",
        "experimental",
    ]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ChatRequest:
        """Create an instance of ChatRequest from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in search_services (list)
        _items = []
        if self.search_services:
            for _item in self.search_services:
                if _item:
                    _items.append(_item.to_dict())
            _dict["search_services"] = _items

        # override the default output from pydantic by calling `to_dict()` of each item in semantic_models (list)
        _items = []
        if self.semantic_models:
            for _item in self.semantic_models:
                if _item:
                    _items.append(_item.to_dict())
            _dict["semantic_models"] = _items

        # override the default output from pydantic by calling `to_dict()` of each item in messages (list)
        _items = []
        if self.messages:
            for _item in self.messages:
                if _item:
                    _items.append(_item.to_dict())
            _dict["messages"] = _items

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ChatRequest:
        """Create an instance of ChatRequest from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ChatRequest.model_validate(obj)

        _obj = ChatRequest.model_validate(
            {
                "query": obj.get("query"),
                "model": obj.get("model"),
                "search_services": [
                    ChatRequestSearchServicesInner.from_dict(_item) for _item in obj.get("search_services")
                ]
                if obj.get("search_services") is not None
                else None,
                "semantic_models": [
                    ChatRequestSemanticModelsInner.from_dict(_item) for _item in obj.get("semantic_models")
                ]
                if obj.get("semantic_models") is not None
                else None,
                "prompt": obj.get("prompt"),
                "messages": [ChatRequestMessagesInner.from_dict(_item) for _item in obj.get("messages")]
                if obj.get("messages") is not None
                else None,
                "debug": obj.get("debug"),
                "max_results": obj.get("max_results"),
                "experimental": obj.get("experimental"),
            }
        )

        return _obj


class ChatRequestModel:
    def __init__(
        self,
        query: str,
        # optional properties
        model: Optional[str] = None,
        search_services: Optional[list[ChatRequestSearchServicesInner]] = None,
        semantic_models: Optional[list[ChatRequestSemanticModelsInner]] = None,
        prompt: Optional[str] = None,
        messages: Optional[list[ChatRequestMessagesInner]] = None,
        debug: Optional[bool] = None,
        max_results: Optional[int] = None,
        experimental: Optional[object] = None,
    ):
        """A model object representing the ChatRequest resource.

        Constructs an object of type ChatRequest with the provided properties.

        Parameters
        __________
        query : str
            Unstructured text query.
        model : str, optional
            The model to be used for the chat.
        search_services : list[ChatRequestSearchServicesInner], optional
            List of search services.
        semantic_models : list[ChatRequestSemanticModelsInner], optional
            The semantic models to be used for Cortex Analyst
        prompt : str, optional
            Additional prompt text.
        messages : list[ChatRequestMessagesInner], optional
            List of conversation messages.
        debug : bool, optional
            Whether to return debug information.
        max_results : int, optional
            Global max number of results from search for llm generation. If not set, default to sum of max_results across list of search_service.
        experimental : object, optional
            reserved
        """
        self.query = query
        self.model = model
        self.search_services = search_services
        self.semantic_models = semantic_models
        self.prompt = prompt
        self.messages = messages
        self.debug = debug
        self.max_results = max_results
        self.experimental = experimental

    __properties = [
        "query",
        "model",
        "search_services",
        "semantic_models",
        "prompt",
        "messages",
        "debug",
        "max_results",
        "experimental",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ChatRequest(
            query=self.query,
            model=self.model,
            search_services=[x._to_model() for x in self.search_services] if self.search_services is not None else None,
            semantic_models=[x._to_model() for x in self.semantic_models] if self.semantic_models is not None else None,
            prompt=self.prompt,
            messages=[x._to_model() for x in self.messages] if self.messages is not None else None,
            debug=self.debug,
            max_results=self.max_results,
            experimental=self.experimental,
        )

    @classmethod
    def _from_model(cls, model) -> ChatRequestModel:
        return ChatRequestModel(
            query=model.query,
            model=model.model,
            search_services=[ChatRequestSearchServicesInnerModel._from_model(x) for x in model.search_services]
            if model.search_services
            else None,
            semantic_models=[ChatRequestSemanticModelsInnerModel._from_model(x) for x in model.semantic_models]
            if model.semantic_models
            else None,
            prompt=model.prompt,
            messages=[ChatRequestMessagesInnerModel._from_model(x) for x in model.messages] if model.messages else None,
            debug=model.debug,
            max_results=model.max_results,
            experimental=model.experimental,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ChatRequest.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ChatRequest object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ChatRequestModel:
        """Create an instance of ChatRequest from a dict.

        This method constructs a ChatRequest object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ChatRequest
            A ChatRequest object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ChatRequest.from_dict(obj))


ChatRequest._model_class = ChatRequestModel



"""
Cortex Analyst API.

The Snowflake Cortex Analyst API is a REST API that allows end user to chat with their data leveraging semantic models to generate SQL queries.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictStr

from snowflake.core.cortex.analyst_service._generated.models.tableau_integration import (
    TableauIntegration,
    TableauIntegrationModel,
)


class TableauServerFiles(BaseModel):
    """A model object representing the TableauServerFiles resource.

    Constructs an object of type TableauServerFiles with the provided properties.

    Parameters
    __________
    tableau_urls : list[str], optional

    integration : TableauIntegration, optional
    """

    tableau_urls: Optional[List[StrictStr]] = None

    integration: Optional[TableauIntegration] = None

    __properties = ["tableau_urls", "integration"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TableauServerFiles:
        """Create an instance of TableauServerFiles from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of integration
        if self.integration:
            _dict["integration"] = self.integration.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> TableauServerFiles:
        """Create an instance of TableauServerFiles from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return TableauServerFiles.model_validate(obj)

        _obj = TableauServerFiles.model_validate(
            {
                "tableau_urls": obj.get("tableau_urls"),
                "integration": TableauIntegration.from_dict(obj.get("integration"))
                if obj.get("integration") is not None
                else None,
            }
        )

        return _obj


class TableauServerFilesModel:
    def __init__(
        self,  # optional properties
        tableau_urls: Optional[list[str]] = None,
        integration: Optional[TableauIntegration] = None,
    ):
        """A model object representing the TableauServerFiles resource.

        Constructs an object of type TableauServerFiles with the provided properties.

        Parameters
        __________
        tableau_urls : list[str], optional

        integration : TableauIntegration, optional
        """
        self.tableau_urls = tableau_urls
        self.integration = integration

    __properties = ["tableau_urls", "integration"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return TableauServerFiles(
            tableau_urls=self.tableau_urls,
            integration=self.integration._to_model() if self.integration is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> TableauServerFilesModel:
        return TableauServerFilesModel(
            tableau_urls=model.tableau_urls,
            integration=TableauIntegrationModel._from_model(model.integration) if model.integration else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a TableauServerFiles.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the TableauServerFiles object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> TableauServerFilesModel:
        """Create an instance of TableauServerFiles from a dict.

        This method constructs a TableauServerFiles object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        TableauServerFiles
            A TableauServerFiles object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(TableauServerFiles.from_dict(obj))


TableauServerFiles._model_class = TableauServerFilesModel



"""
Cortex Search REST API.

OpenAPI 3.0 specification for the Cortex Search REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr


class SuggestQueryRequest(BaseModel):
    """A model object representing the SuggestQueryRequest resource.

    Constructs an object of type SuggestQueryRequest with the provided properties.

    Parameters
    __________
    query : str
        The text query on which to suggest.
    columns : list[str], optional
        List of columns to return.
    limit : int,  default 5
        Max number of results to return.
    """

    query: StrictStr

    columns: Optional[List[StrictStr]] = None

    limit: Optional[StrictInt] = 5

    __properties = ["query", "columns", "limit"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SuggestQueryRequest:
        """Create an instance of SuggestQueryRequest from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> SuggestQueryRequest:
        """Create an instance of SuggestQueryRequest from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return SuggestQueryRequest.model_validate(obj)

        _obj = SuggestQueryRequest.model_validate(
            {
                "query": obj.get("query"),
                "columns": obj.get("columns"),
                "limit": obj.get("limit") if obj.get("limit") is not None else 5,
            }
        )

        return _obj


class SuggestQueryRequestModel:
    def __init__(
        self,
        query: str,
        # optional properties
        columns: Optional[list[str]] = None,
        limit: Optional[int] = 5,
    ):
        """A model object representing the SuggestQueryRequest resource.

        Constructs an object of type SuggestQueryRequest with the provided properties.

        Parameters
        __________
        query : str
            The text query on which to suggest.
        columns : list[str], optional
            List of columns to return.
        limit : int,  default 5
            Max number of results to return.
        """
        self.query = query
        self.columns = columns
        self.limit = limit

    __properties = ["query", "columns", "limit"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return SuggestQueryRequest(
            query=self.query,
            columns=self.columns,
            limit=self.limit,
        )

    @classmethod
    def _from_model(cls, model) -> SuggestQueryRequestModel:
        return SuggestQueryRequestModel(
            query=model.query,
            columns=model.columns,
            limit=model.limit,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a SuggestQueryRequest.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the SuggestQueryRequest object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> SuggestQueryRequestModel:
        """Create an instance of SuggestQueryRequest from a dict.

        This method constructs a SuggestQueryRequest object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        SuggestQueryRequest
            A SuggestQueryRequest object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(SuggestQueryRequest.from_dict(obj))


SuggestQueryRequest._model_class = SuggestQueryRequestModel



import pytest

from snowflake.core.event_table import EventTable
from snowflake.core.exceptions import NotFoundError
from tests.utils import random_string


@pytest.mark.min_sf_ver("8.35.0")
def test_rename(event_tables):
    event_table_name = random_string(10, "test_original_event_table_")
    event_table_other_name = random_string(10, "test_other_event_table_")

    event_table_handle = event_tables.create(EventTable(name=event_table_name))

    event_table_handle.fetch()

    event_table_handle.rename(event_table_other_name)

    assert event_table_handle.fetch().name.upper() == event_table_other_name.upper()

    with pytest.raises(NotFoundError):
        event_tables[event_table_name].fetch()

    with pytest.raises(NotFoundError):
        event_tables[event_table_name].rename(event_table_other_name)

    event_tables[event_table_name].rename(event_table_other_name, if_exists=True)
    event_table_handle.drop(if_exists=True)



import pytest


pytestmark = [pytest.mark.min_sf_ver("8.35.0")]


def test_fetch_pipe(pipes, template_pipe):
    try:
        pipe_resource = pipes.create(template_pipe)
        fetched_pipe = pipe_resource.fetch()
        assert fetched_pipe.name == template_pipe.name.upper()
        assert fetched_pipe.comment == template_pipe.comment
        assert fetched_pipe.copy_statement == template_pipe.copy_statement
        assert not fetched_pipe.auto_ingest
        assert fetched_pipe.created_on is not None
        assert fetched_pipe.database_name is not None
        assert fetched_pipe.schema_name is not None
        assert fetched_pipe.owner is not None
        assert fetched_pipe.owner_role_type is not None

        # create pipe with the same name but with or_replace
        template_pipe.comment = "new comment"
        pipe_resource = pipes.create(template_pipe, mode="or_replace")
        fetched_pipe = pipe_resource.fetch()
        assert fetched_pipe.comment == template_pipe.comment
    finally:
        pipe_resource.drop(if_exists=True)



import typing

import pytest

from snowflake.core.user import User, UserCollection
from tests.utils import random_string


if typing.TYPE_CHECKING:
    from snowflake.snowpark import Session


# TODO: SNOW-1542604 Grant accountadmin to Jenkins account
# pytestmark = pytest.mark.jenkins


@pytest.mark.snowpark
@pytest.mark.use_accountadmin
@pytest.mark.min_sf_ver("8.32.0")
def test_create_or_alter_user(users: UserCollection, session: "Session"):
    user_name = random_string(5, "test_create_user_1")
    test_user_basic = User(name=user_name, password="test")
    test_user_props = User(
        name=user_name,
        display_name="test_name",
        first_name="firstname",
        last_name="lastname",
        email="test@snowflake.com",
        must_change_password=False,
        disabled=False,
        days_to_expiry=1,
        mins_to_unlock=10,
        mins_to_bypass_mfa=60,
        default_warehouse="test",
        default_namespace="test",
        default_role="public",
        default_secondary_roles="ALL",
        comment="test_comment",
    )

    try:
        users[user_name].create_or_alter(test_user_basic)

        user = users[user_name].fetch()

        assert user.name.lower() == user_name
        assert user.first_name is None
        assert user.last_name is None
        assert user.email is None
        assert user.created_on is not None
        assert user.comment is None

        users[user_name].create_or_alter(test_user_props)

        user = users[user_name].fetch()
        assert user.name.lower() == user_name
        assert user.first_name.lower() == "firstname"
        assert user.last_name.lower() == "lastname"
        assert user.email.lower() == "test@snowflake.com"
        assert user.created_on is not None
        assert user.comment.lower() == "test_comment"

        users[user_name].drop()
    finally:
        session.sql(f"DROP USER IF EXISTS {user_name}").collect()



import copy

import pytest

from snowflake.core.exceptions import NotFoundError
from tests.integ.utils import random_string

from .conftest import test_sequence_template


def test_drop(sequences):
    prefix = "test_sequence_drop_"
    pre_create_count = len(list(sequences.iter(like=prefix + "%")))

    name = random_string(10, prefix)
    sequence_handle = sequences[name]

    sequence = copy.deepcopy(test_sequence_template)
    sequence.name = name
    sequences.create(sequence)

    created_count = len(list(sequences.iter(like=prefix + "%")))
    assert pre_create_count + 1 == created_count

    sequence_handle.drop()

    after_drop_count = len(list(sequences.iter(like=prefix + "%")))
    assert pre_create_count == after_drop_count

    with pytest.raises(NotFoundError):
        sequence_handle.drop()
        sequence_handle.drop(if_exists=False)

    sequence_handle.drop(if_exists=True)



import copy
import time

import pytest

from snowflake.core import Clone, CreateMode, PointOfTimeOffset
from snowflake.core.exceptions import APIError, ConflictError
from snowflake.core.schema import Schema, SchemaCollection
from tests.utils import random_string, unquote


pytestmark = pytest.mark.usefixtures("backup_database_schema")


def test_create_schema(schemas: SchemaCollection):
    new_schema_def = Schema(name=random_string(10, "test_schema_int_test_"), kind="TRANSIENT")
    new_schema_def.comment = "schema first"
    schema = schemas.create(new_schema_def)
    try:
        created_schema = schema.fetch()
        assert created_schema.name == new_schema_def.name.upper()
        assert created_schema.kind == "TRANSIENT"
        assert created_schema.comment == new_schema_def.comment
        assert created_schema.options != "MANAGED ACCESS"

        with pytest.raises(ConflictError):
            schemas.create(new_schema_def, mode=CreateMode.error_if_exists)

        new_schema_def_1 = copy.deepcopy(new_schema_def)
        new_schema_def_1.kind = None
        new_schema_def_1.comment = "schema second"
        schema = schemas.create(new_schema_def_1, mode=CreateMode.if_not_exists)

        created_schema = schema.fetch()
        assert created_schema.name == new_schema_def.name.upper()
        assert created_schema.kind == "TRANSIENT"
        assert created_schema.comment == new_schema_def.comment
        assert created_schema.options != "MANAGED ACCESS"
    finally:
        schema.drop()

    schema = schemas.create(new_schema_def_1, mode=CreateMode.or_replace)
    try:
        created_schema = schema.fetch()
        assert created_schema.name == new_schema_def_1.name.upper()
        assert created_schema.kind == "PERMANENT"
        assert created_schema.comment == new_schema_def_1.comment
    finally:
        schema.drop()

    schema_name = random_string(10, "test_schema_INT_test_")
    schema_name_case_sensitive = '"' + schema_name + '"'
    new_schema_def = Schema(name=schema_name_case_sensitive)
    schema = schemas.create(new_schema_def)
    try:
        created_schema = schema.fetch()
        assert created_schema.name == unquote(new_schema_def.name)
    finally:
        schema.drop()


def test_create_with_managed_access(schemas: SchemaCollection):
    new_schema_def = Schema(name=random_string(10, "test_schema_int_test_"), managed_access=True)
    try:
        schema = schemas.create(new_schema_def, mode=CreateMode.or_replace)

        created_schema = schema.fetch()
        assert created_schema.name == new_schema_def.name.upper()
        assert created_schema.managed_access is True
        assert created_schema.options == "MANAGED ACCESS"
    finally:
        schema.drop()


@pytest.mark.min_sf_ver("9.8.0")
def test_create_clone(schemas: SchemaCollection):
    schema_name = random_string(10, "test_schema_")
    schema_def = Schema(name=schema_name, kind="TRANSIENT")

    new_schema_name = random_string(10, "test_schema_clone")
    new_schema_def = Schema(name=new_schema_name)

    # error because Schema does not exist
    with pytest.raises(APIError, match="does not exist"):
        schemas.create(
            new_schema_def,
            clone=Clone(source=schema_name, point_of_time=PointOfTimeOffset(reference="at", when="-5")),
            mode=CreateMode.or_replace,
        )

    schemas.create(schema_def)
    # error because transient schema cannot be cloned to a permanent schema
    with pytest.raises(APIError, match="transient object cannot be cloned to a permanent object"):
        schemas.create(
            new_schema_def,
            clone=Clone(source=schema_name, point_of_time=PointOfTimeOffset(reference="at", when="-1")),
            mode=CreateMode.or_replace,
        )

    # can clone transient to transient
    new_schema_def.kind = "TRANSIENT"
    # sleep is needed due to using point_of_time in clone
    time.sleep(2)
    schemas.create(
        new_schema_def,
        clone=Clone(source=schema_name, point_of_time=PointOfTimeOffset(reference="at", when="-1")),
        mode=CreateMode.or_replace,
    )

    # replaced transient to permanent schema
    schema_def.kind = new_schema_def.kind = None
    schemas.create(schema_def, mode=CreateMode.or_replace)
    schema = schemas.create(new_schema_def, clone=Clone(source=schema_name), mode=CreateMode.or_replace)
    try:
        schema.fetch()
    finally:
        schema.drop()

    # clone schema setting optional attributes on the clone
    schema_def.kind = new_schema_def.kind = None
    new_schema_def.serverless_task_min_statement_size = "SMALL"
    new_schema_def.serverless_task_max_statement_size = "MEDIUM"
    new_schema_def.user_task_managed_initial_warehouse_size = "SMALL"
    new_schema_def.suspend_task_after_num_failures = 2
    new_schema_def.user_task_timeout_ms = 20000
    schemas.create(schema_def, mode=CreateMode.or_replace)
    schema = schemas.create(new_schema_def, clone=Clone(source=schema_name), mode=CreateMode.or_replace)
    try:
        fetched_schema = schema.fetch()
        assert fetched_schema.serverless_task_min_statement_size == "SMALL"
        assert fetched_schema.serverless_task_max_statement_size == "MEDIUM"
        assert fetched_schema.user_task_managed_initial_warehouse_size == "SMALL"
        assert fetched_schema.suspend_task_after_num_failures == 2
        assert fetched_schema.user_task_timeout_ms == 20000
    finally:
        schema.drop()


def test_create_clone_cross_database(schemas: SchemaCollection, temp_db):
    # clone temp_db.original_schema to <current database>.clone_schema
    original_schema_def = Schema(name=random_string(10, "test_schema_cross_database_"))
    original_schema_def.comment = "original schema"

    original_schema = temp_db.schemas.create(original_schema_def)

    clone_of_schema_in_new_db = Clone(source=f"{temp_db.name}.{original_schema_def.name}")

    clone_schema = schemas.create(
        Schema(name=random_string(10, "test_schema_cross_database_")), clone=clone_of_schema_in_new_db
    )

    try:
        cloned_schema_def = clone_schema.fetch()
        original_schema_def = original_schema.fetch()

        assert original_schema_def.database_name.upper() == temp_db.name.upper()
        assert original_schema_def.comment.upper() == "original schema".upper()
        assert cloned_schema_def.database_name.upper() == schemas.database.name.upper()
        assert cloned_schema_def.comment.upper() == "original schema".upper()

        assert original_schema_def.name.upper() in [i.name.upper() for i in temp_db.schemas.iter()]
        assert cloned_schema_def.name.upper() in [i.name.upper() for i in schemas.iter()]
    finally:
        clone_schema.drop()
        original_schema.drop()



"""
Snowflake Image Repository API.

The Snowflake Image Repository API is a REST API that you can use to access, update, and perform common actions on Image Repository resource in Snowflake.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing_extensions import Annotated


class ImageRepository(BaseModel):
    """A model object representing the ImageRepository resource.

    Constructs an object of type ImageRepository with the provided properties.

    Parameters
    __________
    name : str
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    database_name : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    schema_name : str, optional
        A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
    created_on : datetime, optional
        Time the image repository was created — **Read-only:** *any user-provided value will be ignored.*
    repository_url : str, optional
        Current URL of the image repository — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Identifier for the current owner of the image repository — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        Role type of the image repository owner — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Annotated[str, Field(strict=True)]

    database_name: Optional[Annotated[str, Field(strict=True)]] = None

    schema_name: Optional[Annotated[str, Field(strict=True)]] = None

    created_on: Optional[datetime] = None

    repository_url: Optional[StrictStr] = None

    owner: Optional[StrictStr] = None

    owner_role_type: Optional[StrictStr] = None

    __properties = ["name", "database_name", "schema_name", "created_on", "repository_url", "owner", "owner_role_type"]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("database_name")
    def database_name_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("schema_name")
    def schema_name_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ImageRepository:
        """Create an instance of ImageRepository from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "repository_url",
                    "owner",
                    "owner_role_type",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> ImageRepository:
        """Create an instance of ImageRepository from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return ImageRepository.model_validate(obj)

        _obj = ImageRepository.model_validate(
            {
                "name": obj.get("name"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
                "created_on": obj.get("created_on"),
                "repository_url": obj.get("repository_url"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
            }
        )

        return _obj


class ImageRepositoryModel:
    def __init__(
        self,
        name: str,
        # optional properties
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
        created_on: Optional[datetime] = None,
        repository_url: Optional[str] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
    ):
        """A model object representing the ImageRepository resource.

        Constructs an object of type ImageRepository with the provided properties.

        Parameters
        __________
        name : str
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        database_name : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        schema_name : str, optional
            A Snowflake object identifier. If the identifier contains spaces or special characters, the entire string must be enclosed in double quotes. Identifiers enclosed in double quotes are also case-sensitive.
        created_on : datetime, optional
            Time the image repository was created.
        repository_url : str, optional
            Current URL of the image repository.
        owner : str, optional
            Identifier for the current owner of the image repository.
        owner_role_type : str, optional
            Role type of the image repository owner.
        """
        self.name = name
        self.database_name = database_name
        self.schema_name = schema_name
        self.created_on = created_on
        self.repository_url = repository_url
        self.owner = owner
        self.owner_role_type = owner_role_type

    __properties = ["name", "database_name", "schema_name", "created_on", "repository_url", "owner", "owner_role_type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return ImageRepository(
            name=self.name,
            database_name=self.database_name,
            schema_name=self.schema_name,
            created_on=self.created_on,
            repository_url=self.repository_url,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
        )

    @classmethod
    def _from_model(cls, model) -> ImageRepositoryModel:
        return ImageRepositoryModel(
            name=model.name,
            database_name=model.database_name,
            schema_name=model.schema_name,
            created_on=model.created_on,
            repository_url=model.repository_url,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a ImageRepository.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the ImageRepository object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ImageRepositoryModel:
        """Create an instance of ImageRepository from a dict.

        This method constructs a ImageRepository object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        ImageRepository
            A ImageRepository object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(ImageRepository.from_dict(obj))


ImageRepository._model_class = ImageRepositoryModel



# coding: utf-8

# flake8: noqa
"""
Preview Feature Test API

Test spec for preview feature filtering  # noqa: E501

The version of the OpenAPI document: 1.0.0
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

# import models into model package
from tests.preview_test._generated.models.create_resource_request import CreateResourceRequest
from tests.preview_test._generated.models.resource import Resource
from tests.preview_test._generated.models.shared_model import SharedModel

__all__ = [
    "CreateResourceRequest",
    "Resource",
    "SharedModel",
]



"""
Snowflake API Integration API.

The Snowflake API Integration API is a REST API that you can use to access, update, and perform certain actions on API Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, StrictStr, field_validator

from snowflake.core.api_integration._generated.models.api_hook import ApiHook


class AzureHook(ApiHook):
    """A model object representing the AzureHook resource.

    Constructs an object of type AzureHook with the provided properties.

    Parameters
    __________
    api_provider : str

    azure_tenant_id : str
        The ID of the Office 365 tenant that all Azure API Management instances belong to.
    azure_ad_application_id : str
        The application ID of the Azure Active Directory application.
    api_key : str, optional
        An alphanumeric string that is used to identify API clients and control access to the API, also called a subscription key.
    """

    api_provider: StrictStr

    azure_tenant_id: StrictStr

    azure_ad_application_id: StrictStr

    api_key: Optional[StrictStr] = None

    __properties = ["type"]

    @field_validator("api_provider")
    def api_provider_validate_enum(cls, v):
        if v not in ("AZURE_API_MANAGEMENT", "AZURE_PRIVATE_API_MANAGEMENT"):
            raise ValueError("must validate the enum values ('AZURE_API_MANAGEMENT','AZURE_PRIVATE_API_MANAGEMENT')")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AzureHook:
        """Create an instance of AzureHook from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # set to None if api_key (nullable) is None
        if self.api_key is None:
            _dict["api_key"] = None

        _dict["type"] = ApiHook.get_child_model_discriminator_value("AzureHook")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> AzureHook:
        """Create an instance of AzureHook from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return AzureHook.model_validate(obj)

        _obj = AzureHook.model_validate(
            {
                "api_provider": obj.get("api_provider"),
                "azure_tenant_id": obj.get("azure_tenant_id"),
                "azure_ad_application_id": obj.get("azure_ad_application_id"),
                "api_key": obj.get("api_key"),
            }
        )

        return _obj


class AzureHookModel(ApiHook):
    def __init__(
        self,
        api_provider: str,
        azure_tenant_id: str,
        azure_ad_application_id: str,
        # optional properties
        api_key: Optional[str] = None,
    ):
        """A model object representing the AzureHook resource.

        Constructs an object of type AzureHook with the provided properties.

        Parameters
        __________
        api_provider : str

        azure_tenant_id : str
            The ID of the Office 365 tenant that all Azure API Management instances belong to.
        azure_ad_application_id : str
            The application ID of the Azure Active Directory application.
        api_key : str, optional
            An alphanumeric string that is used to identify API clients and control access to the API, also called a subscription key.
        """
        super().__init__()
        self.api_provider = api_provider
        self.azure_tenant_id = azure_tenant_id
        self.azure_ad_application_id = azure_ad_application_id
        self.api_key = api_key

    __properties = ["type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return AzureHook(
            api_provider=self.api_provider,
            azure_tenant_id=self.azure_tenant_id,
            azure_ad_application_id=self.azure_ad_application_id,
            api_key=self.api_key,
        )

    @classmethod
    def _from_model(cls, model) -> AzureHookModel:
        return AzureHookModel(
            api_provider=model.api_provider,
            azure_tenant_id=model.azure_tenant_id,
            azure_ad_application_id=model.azure_ad_application_id,
            api_key=model.api_key,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a AzureHook.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the AzureHook object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> AzureHookModel:
        """Create an instance of AzureHook from a dict.

        This method constructs a AzureHook object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        AzureHook
            A AzureHook object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(AzureHook.from_dict(obj))


AzureHook._model_class = AzureHookModel



"""
Cortex Analyst API.

The Snowflake Cortex Analyst API is a REST API that allows end user to chat with their data leveraging semantic models to generate SQL queries.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any

from pydantic import BaseModel, ConfigDict

from snowflake.core.cortex.analyst_service._generated.models.staging_data import StagingData, StagingDataModel
from snowflake.core.cortex.analyst_service._generated.models.tableau_auth import TableauAuth, TableauAuthModel


class TableauIntegration(BaseModel):
    """A model object representing the TableauIntegration resource.

    Constructs an object of type TableauIntegration with the provided properties.

    Parameters
    __________
    auth : TableauAuth

    staging_data : StagingData
    """

    auth: TableauAuth

    staging_data: StagingData

    __properties = ["auth", "staging_data"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> TableauIntegration:
        """Create an instance of TableauIntegration from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of auth
        if self.auth:
            _dict["auth"] = self.auth.to_dict()

        # override the default output from pydantic by calling `to_dict()` of staging_data
        if self.staging_data:
            _dict["staging_data"] = self.staging_data.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> TableauIntegration:
        """Create an instance of TableauIntegration from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return TableauIntegration.model_validate(obj)

        _obj = TableauIntegration.model_validate(
            {
                "auth": TableauAuth.from_dict(obj.get("auth")) if obj.get("auth") is not None else None,
                "staging_data": StagingData.from_dict(obj.get("staging_data"))
                if obj.get("staging_data") is not None
                else None,
            }
        )

        return _obj


class TableauIntegrationModel:
    def __init__(
        self,
        auth: TableauAuth,
        staging_data: StagingData,
        # optional properties
    ):
        """A model object representing the TableauIntegration resource.

        Constructs an object of type TableauIntegration with the provided properties.

        Parameters
        __________
        auth : TableauAuth

        staging_data : StagingData
        """
        self.auth = auth
        self.staging_data = staging_data

    __properties = ["auth", "staging_data"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return TableauIntegration(
            auth=self.auth._to_model() if self.auth is not None else None,
            staging_data=self.staging_data._to_model() if self.staging_data is not None else None,
        )

    @classmethod
    def _from_model(cls, model) -> TableauIntegrationModel:
        return TableauIntegrationModel(
            auth=TableauAuthModel._from_model(model.auth) if model.auth else None,
            staging_data=StagingDataModel._from_model(model.staging_data) if model.staging_data else None,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a TableauIntegration.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the TableauIntegration object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> TableauIntegrationModel:
        """Create an instance of TableauIntegration from a dict.

        This method constructs a TableauIntegration object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        TableauIntegration
            A TableauIntegration object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(TableauIntegration.from_dict(obj))


TableauIntegration._model_class = TableauIntegrationModel



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.image_repository._generated.api.image_repository_api import ImageRepositoryApi

__all__ = [
    "ImageRepositoryApi",
]



"""
Snowflake External Volume API.

The Snowflake External Volume API is a REST API that you can use to access, update, and perform certain actions on External Volume resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from typing import TYPE_CHECKING, Iterator, Optional, TypeVar, Union

from snowflake.core._common import AccountObjectCollectionParent, CreateMode, ObjectReferenceMixin
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core._operation import PollingOperation, PollingOperations
from snowflake.core._utils import get_function_name_with_args, replace_function_name_in_name_with_args
from snowflake.core.external_volume._generated.models.external_volume import ExternalVolume

from .external_volume_api import ExternalVolumeApi


if TYPE_CHECKING:
    from snowflake.core import Root


T = TypeVar("T")


class ExternalVolumeCollectionBase(AccountObjectCollectionParent["ExternalVolumeResource"]):
    _identifier_requires_args: bool = False

    def __init__(self, root: "Root", resource_class: type[T]) -> None:
        super().__init__(root, resource_class)
        self._api = ExternalVolumeApi(
            root=self.root, resource_class=self._ref_class, sproc_client=StoredProcApiClient(root=self.root)
        )

    @api_telemetry
    def create(
        self, external_volume: ExternalVolume, mode: Optional[Union[CreateMode, str]] = None
    ) -> "ExternalVolumeResource":  # noqa: F821
        """Create an external volume.

        Parameters
        __________
        external_volume: ExternalVolume
             (required)
        mode: Union[CreateMode, str]
             Parameter allowing support for different modes of resource creation. Possible values include: - `errorIfExists`: Throws an error if you try to create a resource that already exists. - `orReplace`: Automatically replaces the existing resource with the current one. - `ifNotExists`: Creates a new resource when an alter is requested for a non-existent resource.
        """
        if isinstance(mode, str):
            mode = CreateMode[mode].value

        self._api.create_external_volume(
            external_volume=external_volume,
            create_mode=mode,
            async_req=False,
        )
        if self._identifier_requires_args:
            return self._ref_class(get_function_name_with_args(external_volume), self)
        return self._ref_class(external_volume.name, self)

    @api_telemetry
    def create_async(
        self, external_volume: ExternalVolume, mode: Optional[Union[CreateMode, str]] = None
    ) -> PollingOperation["ExternalVolumeResource"]:  # noqa: F821
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """
        if isinstance(mode, str):
            mode = CreateMode[mode].value

        future = self._api.create_external_volume(
            external_volume=external_volume,
            create_mode=mode,
            async_req=True,
        )
        if self._identifier_requires_args:
            return PollingOperation(
                future, lambda _: self._ref_class(get_function_name_with_args(external_volume), self)
            )
        return PollingOperation(
            future,
            lambda _: self._ref_class(
                external_volume if isinstance(external_volume, str) else external_volume.name, self
            ),
        )

    @api_telemetry
    def iter(self, *, like: Optional[str] = None) -> Iterator[ExternalVolume]:
        """List external volumes.

        Parameters
        __________
        like: str
             Parameter to filter the command output by resource name. Uses case-insensitive pattern matching, with support for SQL wildcard characters.
        """
        resources = self._api.list_external_volumes(
            like=like,
            async_req=False,
        )
        return iter(resources)

    @api_telemetry
    def iter_async(self, *, like: Optional[str] = None) -> PollingOperation[Iterator[ExternalVolume]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """
        future = self._api.list_external_volumes(
            like=like,
            async_req=True,
        )

        return PollingOperations.iterator(future)


class ExternalVolumeResourceBase(ObjectReferenceMixin["ExternalVolumeCollection"]):
    _collection_class: ExternalVolumeCollectionBase
    _identifier_requires_args: bool = False
    _plural_name: str

    def __init__(self, name: str, collection_class: ExternalVolumeCollectionBase) -> None:
        self.collection = collection_class
        if self._identifier_requires_args:
            self.name_with_args = name
        else:
            self.name = name

    @api_telemetry
    def fetch(
        self,
    ) -> ExternalVolume:
        """Fetch an external volume.

        Parameters
        __________
        """
        result = self.collection._api.fetch_external_volume(
            self._identifier,
            async_req=False,
        )
        return result

    @api_telemetry
    def fetch_async(
        self,
    ) -> PollingOperation[ExternalVolume]:
        """An asynchronous version of :func:`fetch`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """
        future = self.collection._api.fetch_external_volume(
            self._identifier,
            async_req=True,
        )

        return PollingOperations.identity(future)

    @api_telemetry
    def undrop(
        self,
    ) -> None:
        """Undrop an external volume.

        Parameters
        __________
        """
        self.collection._api.undrop_external_volume(
            self._identifier,
            async_req=False,
        )

    @api_telemetry
    def undrop_async(
        self,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`undrop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """
        future = self.collection._api.undrop_external_volume(
            self._identifier,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def drop(
        self,
        if_exists: Optional[bool] = None,
    ) -> None:
        """Delete an external volume.

        Parameters
        __________
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`: The endpoint does not throw an error if the resource does not exist. It returns a 200 success response, but does not take any action on the resource. - `false`: The endpoint throws an error if the resource doesn't exist.
        """
        self.collection._api.delete_external_volume(
            self._identifier,
            if_exists=if_exists,
            async_req=False,
        )

    @api_telemetry
    def drop_async(
        self,
        if_exists: Optional[bool] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`drop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """
        future = self.collection._api.delete_external_volume(
            self._identifier,
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @property
    def _identifier(self) -> str:
        """Get the identifier for this resource."""
        if self._identifier_requires_args:
            return self.name_with_args
        return self.name

    def _set_new_name(self, target_name: str) -> None:
        """Set a new name for the resource."""
        if self._identifier_requires_args:
            old_name = self.name_with_args
            self.name_with_args = replace_function_name_in_name_with_args(self.name_with_args, target_name)
        else:
            old_name = self.name
            self.name = target_name

        # Update the collection reference to point to the new name
        self.collection.update_reference(old_name=old_name, new_name=self._identifier, resource=self)



from unittest import mock

import pytest

from snowflake.core import PollingOperation
from snowflake.core.alert import Alert, AlertResource, MinutesSchedule
from snowflake.core.alert._generated import TagAssignment, TagReference
from snowflake.core.tag import TagValue

from ...utils import BASE_URL, extra_params, mock_http_response


API_CLIENT_REQUEST = "snowflake.core._generated.api_client.ApiClient.request"
ALERT = Alert(name="my_alert", schedule=MinutesSchedule(minutes=60), condition="", action="select 1")


@pytest.fixture()
def alerts(schema):
    return schema.alerts


@pytest.fixture()
def alert(alerts):
    return alerts["my_alert"]


def test_create_async(fake_root, alerts):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/alerts?createMode=errorIfExists")
    kwargs = extra_params(
        query_params=[("createMode", "errorIfExists")],
        body={
            "name": "my_alert",
            "schedule": {"minutes": 60, "schedule_type": "SCHEDULE_TYPE"},
            "condition": "",
            "action": "select 1",
        },
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        alert_res = alerts.create(ALERT)
        assert isinstance(alert_res, AlertResource)
        assert alert_res.name == "my_alert"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = alerts.create_async(ALERT)
        assert isinstance(op, PollingOperation)
        alert_res = op.result()
        assert alert_res.name == "my_alert"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_create_async_clone(fake_root, alerts):
    args = (
        fake_root,
        "POST",
        BASE_URL
        + "/databases/my_db/schemas/my_schema/alerts/clone_alert:clone?"
        + "createMode=errorIfExists&targetDatabase=my_db&targetSchema=my_schema",
    )
    kwargs = extra_params(
        query_params=[("createMode", "errorIfExists"), ("targetDatabase", "my_db"), ("targetSchema", "my_schema")],
        body={"name": "my_alert"},
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        alert_res = alerts.create("my_alert", clone_alert="clone_alert")
        assert alert_res.name == "my_alert"
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = alerts.create_async("my_alert", clone_alert="clone_alert")
        assert isinstance(op, PollingOperation)
        alert_res = op.result()
        assert alert_res.name == "my_alert"
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_iter_async(fake_root, alerts):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/alerts")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        it = alerts.iter()
        assert list(it) == []

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = alerts.iter_async()
        assert isinstance(op, PollingOperation)
        it = op.result()
        assert list(it) == []
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_fetch_async(fake_root, alert):
    from snowflake.core.alert._generated.models import Alert as AlertModel
    from snowflake.core.alert._generated.models import MinutesSchedule as MinutesScheduleModel

    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/alerts/my_alert")
    kwargs = extra_params()
    model = AlertModel(name="my_alert", schedule=MinutesScheduleModel(minutes=60), condition="", action="select 1")

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        my_alert = alert.fetch()
        assert my_alert.to_dict() == ALERT.to_dict()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response(model.to_json())
        op = alert.fetch_async()
        assert isinstance(op, PollingOperation)
        my_alert = op.result()
        assert my_alert.to_dict() == ALERT.to_dict()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_drop_async(fake_root, alert):
    args = (fake_root, "DELETE", BASE_URL + "/databases/my_db/schemas/my_schema/alerts/my_alert?ifExists=False")
    kwargs = extra_params(query_params=[("ifExists", False)])

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        alert.drop()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = alert.drop_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_execute_async(fake_root, alert):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/alerts/my_alert:execute")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        alert.execute()
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = alert.execute_async()
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_set_tags(fake_root, alert, tag):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/alerts/my_alert:set-tags")
    tags = {tag: TagValue(value="value")}
    kwargs = extra_params(
        body=[
            TagAssignment(
                tag_value=v.value, tag_name=k.name, tag_schema=k.schema.name, tag_database=k.database.name
            ).to_dict()
            for k, v in tags.items()
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        alert.set_tags(tags)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = alert.set_tags_async(tags)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_unset_tags(fake_root, alert, tag):
    args = (fake_root, "POST", BASE_URL + "/databases/my_db/schemas/my_schema/alerts/my_alert:unset-tags")
    tag_resources = {tag}
    kwargs = extra_params(
        body=[
            TagReference(
                tag_name=tag_res.name, tag_schema=tag_res.schema.name, tag_database=tag_res.database.name
            ).to_dict()
            for tag_res in tag_resources
        ]
    )

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        alert.unset_tags(tag_resources)
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        op = alert.unset_tags_async(tag_resources)
        assert isinstance(op, PollingOperation)
        op.result()
    mocked_request.assert_called_once_with(*args, **kwargs)


def test_get_tags(fake_root, alert):
    args = (fake_root, "GET", BASE_URL + "/databases/my_db/schemas/my_schema/alerts/my_alert:get-tags")
    kwargs = extra_params()

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        assert alert.get_tags() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)

    with mock.patch(API_CLIENT_REQUEST) as mocked_request:
        mocked_request.return_value = mock_http_response()
        op = alert.get_tags_async()
        assert isinstance(op, PollingOperation)
        assert op.result() == {}
    mocked_request.assert_called_once_with(*args, **kwargs)



"""
Snowflake Procedure API.

The Snowflake Procedure API is a REST API that you can use to access, update, and perform certain actions on Procedure resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re

from datetime import datetime
from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing_extensions import Annotated

from snowflake.core.procedure._generated.models.argument import Argument, ArgumentModel
from snowflake.core.procedure._generated.models.function_language import FunctionLanguage, FunctionLanguageModel
from snowflake.core.procedure._generated.models.return_type import ReturnType, ReturnTypeModel


class Procedure(BaseModel):
    """A model object representing the Procedure resource.

    Constructs an object of type Procedure with the provided properties.

    Parameters
    __________
    name : str
        Name of the procedure
    arguments : list[Argument]
        List of arguments for the function/procedure
    return_type : ReturnType

    language_config : FunctionLanguage

    execute_as : str, optional
        What permissions should the procedure execution be called with
    is_secure : bool, optional
        Specifies whether the function/procedure is secure or not
    comment : str, optional
        Specifies a comment for the function/procedure
    body : str, optional
        Function/procedure definition
    created_on : datetime, optional
        The date and time when the function/procedure was created — **Read-only:** *any user-provided value will be ignored.*
    schema_name : str, optional
        The name of the schema in which the function/procedure exists — **Read-only:** *any user-provided value will be ignored.*
    database_name : str, optional
        The name of the database in which the function/procedure exists — **Read-only:** *any user-provided value will be ignored.*
    min_num_arguments : int, optional
        The minimum number of arguments — **Read-only:** *any user-provided value will be ignored.*
    max_num_arguments : int, optional
        The maximum number of arguments — **Read-only:** *any user-provided value will be ignored.*
    owner : str, optional
        Role that owns the function/procedure — **Read-only:** *any user-provided value will be ignored.*
    owner_role_type : str, optional
        The type of role that owns the function/procedure — **Read-only:** *any user-provided value will be ignored.*
    is_builtin : bool, optional
        If the function/procedure is built-in or not (user-defined) — **Read-only:** *any user-provided value will be ignored.*
    """

    name: Annotated[str, Field(strict=True)]

    execute_as: Optional[StrictStr] = None

    is_secure: Optional[StrictBool] = None

    arguments: List[Argument]

    return_type: ReturnType

    language_config: FunctionLanguage

    comment: Optional[StrictStr] = None

    body: Optional[StrictStr] = None

    created_on: Optional[datetime] = None

    schema_name: Optional[Annotated[str, Field(strict=True)]] = None

    database_name: Optional[Annotated[str, Field(strict=True)]] = None

    min_num_arguments: Optional[StrictInt] = None

    max_num_arguments: Optional[StrictInt] = None

    owner: Optional[StrictStr] = None

    owner_role_type: Optional[StrictStr] = None

    is_builtin: Optional[StrictBool] = None

    __properties = [
        "name",
        "execute_as",
        "is_secure",
        "arguments",
        "return_type",
        "language_config",
        "comment",
        "body",
        "created_on",
        "schema_name",
        "database_name",
        "min_num_arguments",
        "max_num_arguments",
        "owner",
        "owner_role_type",
        "is_builtin",
    ]

    @field_validator("name")
    def name_validate_regular_expression(cls, v):
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("execute_as")
    def execute_as_validate_enum(cls, v):
        if v is None:
            return v
        if v not in ("CALLER", "OWNER"):
            raise ValueError("must validate the enum values ('CALLER','OWNER')")
        return v

    @field_validator("schema_name")
    def schema_name_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    @field_validator("database_name")
    def database_name_validate_regular_expression(cls, v):
        if v is None:
            return v
        if not re.match(r"""^\"([^\"]|\"\")+\"|[a-zA-Z_][a-zA-Z0-9_$]*$""", v):
            raise ValueError(r"""must validate the regular expression /^"([^"]|"")+"|[a-zA-Z_][a-zA-Z0-9_$]*$/""")
        return v

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Procedure:
        """Create an instance of Procedure from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "created_on",
                    "schema_name",
                    "database_name",
                    "min_num_arguments",
                    "max_num_arguments",
                    "owner",
                    "owner_role_type",
                    "is_builtin",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of each item in arguments (list)
        _items = []
        if self.arguments:
            for _item in self.arguments:
                if _item:
                    _items.append(_item.to_dict())
            _dict["arguments"] = _items

        # override the default output from pydantic by calling `to_dict()` of return_type
        if self.return_type:
            _dict["return_type"] = self.return_type.to_dict()

        # override the default output from pydantic by calling `to_dict()` of language_config
        if self.language_config:
            _dict["language_config"] = self.language_config.to_dict()

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Procedure:
        """Create an instance of Procedure from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Procedure.model_validate(obj)

        _obj = Procedure.model_validate(
            {
                "name": obj.get("name"),
                "execute_as": obj.get("execute_as"),
                "is_secure": obj.get("is_secure"),
                "arguments": [Argument.from_dict(_item) for _item in obj.get("arguments")]
                if obj.get("arguments") is not None
                else None,
                "return_type": ReturnType.from_dict(obj.get("return_type"))
                if obj.get("return_type") is not None
                else None,
                "language_config": FunctionLanguage.from_dict(obj.get("language_config"))
                if obj.get("language_config") is not None
                else None,
                "comment": obj.get("comment"),
                "body": obj.get("body"),
                "created_on": obj.get("created_on"),
                "schema_name": obj.get("schema_name"),
                "database_name": obj.get("database_name"),
                "min_num_arguments": obj.get("min_num_arguments"),
                "max_num_arguments": obj.get("max_num_arguments"),
                "owner": obj.get("owner"),
                "owner_role_type": obj.get("owner_role_type"),
                "is_builtin": obj.get("is_builtin"),
            }
        )

        return _obj


class ProcedureModel:
    def __init__(
        self,
        name: str,
        arguments: list[Argument],
        return_type: ReturnType,
        language_config: FunctionLanguage,
        # optional properties
        execute_as: Optional[str] = None,
        is_secure: Optional[bool] = None,
        comment: Optional[str] = None,
        body: Optional[str] = None,
        created_on: Optional[datetime] = None,
        schema_name: Optional[str] = None,
        database_name: Optional[str] = None,
        min_num_arguments: Optional[int] = None,
        max_num_arguments: Optional[int] = None,
        owner: Optional[str] = None,
        owner_role_type: Optional[str] = None,
        is_builtin: Optional[bool] = None,
    ):
        """A model object representing the Procedure resource.

        Constructs an object of type Procedure with the provided properties.

        Parameters
        __________
        name : str
            Name of the procedure
        arguments : list[Argument]
            List of arguments for the function/procedure
        return_type : ReturnType

        language_config : FunctionLanguage

        execute_as : str, optional
            What permissions should the procedure execution be called with
        is_secure : bool, optional
            Specifies whether the function/procedure is secure or not
        comment : str, optional
            Specifies a comment for the function/procedure
        body : str, optional
            Function/procedure definition
        created_on : datetime, optional
            The date and time when the function/procedure was created
        schema_name : str, optional
            The name of the schema in which the function/procedure exists.
        database_name : str, optional
            The name of the database in which the function/procedure exists.
        min_num_arguments : int, optional
            The minimum number of arguments
        max_num_arguments : int, optional
            The maximum number of arguments
        owner : str, optional
            Role that owns the function/procedure
        owner_role_type : str, optional
            The type of role that owns the function/procedure
        is_builtin : bool, optional
            If the function/procedure is built-in or not (user-defined)
        """
        self.name = name
        self.execute_as = execute_as
        self.is_secure = is_secure
        self.arguments = arguments
        self.return_type = return_type
        self.language_config = language_config
        self.comment = comment
        self.body = body
        self.created_on = created_on
        self.schema_name = schema_name
        self.database_name = database_name
        self.min_num_arguments = min_num_arguments
        self.max_num_arguments = max_num_arguments
        self.owner = owner
        self.owner_role_type = owner_role_type
        self.is_builtin = is_builtin

    __properties = [
        "name",
        "execute_as",
        "is_secure",
        "arguments",
        "return_type",
        "language_config",
        "comment",
        "body",
        "created_on",
        "schema_name",
        "database_name",
        "min_num_arguments",
        "max_num_arguments",
        "owner",
        "owner_role_type",
        "is_builtin",
    ]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Procedure(
            name=self.name,
            execute_as=self.execute_as,
            is_secure=self.is_secure,
            arguments=[x._to_model() for x in self.arguments] if self.arguments is not None else None,
            return_type=self.return_type._to_model() if self.return_type is not None else None,
            language_config=self.language_config._to_model() if self.language_config is not None else None,
            comment=self.comment,
            body=self.body,
            created_on=self.created_on,
            schema_name=self.schema_name,
            database_name=self.database_name,
            min_num_arguments=self.min_num_arguments,
            max_num_arguments=self.max_num_arguments,
            owner=self.owner,
            owner_role_type=self.owner_role_type,
            is_builtin=self.is_builtin,
        )

    @classmethod
    def _from_model(cls, model) -> ProcedureModel:
        return ProcedureModel(
            name=model.name,
            execute_as=model.execute_as,
            is_secure=model.is_secure,
            arguments=[ArgumentModel._from_model(x) for x in model.arguments] if model.arguments else None,
            return_type=ReturnTypeModel._from_model(model.return_type) if model.return_type else None,
            language_config=FunctionLanguageModel._from_model(model.language_config) if model.language_config else None,
            comment=model.comment,
            body=model.body,
            created_on=model.created_on,
            schema_name=model.schema_name,
            database_name=model.database_name,
            min_num_arguments=model.min_num_arguments,
            max_num_arguments=model.max_num_arguments,
            owner=model.owner,
            owner_role_type=model.owner_role_type,
            is_builtin=model.is_builtin,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Procedure.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Procedure object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> ProcedureModel:
        """Create an instance of Procedure from a dict.

        This method constructs a Procedure object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Procedure
            A Procedure object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Procedure.from_dict(obj))


Procedure._model_class = ProcedureModel



"""Manages Snowflake Alerts."""

from ..alert._generated.models import Alert, AlertClone, CronSchedule, MinutesSchedule
from ._alert import AlertCollection, AlertResource


__all__ = ["Alert", "AlertResource", "AlertCollection", "AlertClone", "CronSchedule", "MinutesSchedule"]



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.streamlit._generated.api.streamlit_api import StreamlitApi

__all__ = [
    "StreamlitApi",
]



"""
Cortex Embed API.

OpenAPI 3.0 specification for the Cortex Embed REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Annotated


class EmbedRequest(BaseModel):
    """A model object representing the EmbedRequest resource.

    Constructs an object of type EmbedRequest with the provided properties.

    Parameters
    __________
    model : str
        Identifier of the model to use for generating embeddings.
        Refer to Snowflake documentation for the list of supported models.

        Examples:
          - `snowflake-arctic-embed-m`
          - `snowflake-arctic-embed-m-v1.5`
    text : list[str]
        An array of input texts for which vector embeddings will be calculated.
        Example: ["Hello world", "Machine learning is fascinating"]
    provisioned_throughput_id : str, optional
        The provisioned throughput ID to be used with the request.
    """

    model: StrictStr

    text: Annotated[List[StrictStr], Field(min_length=1)]

    provisioned_throughput_id: Optional[StrictStr] = None

    __properties = ["model", "text", "provisioned_throughput_id"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> EmbedRequest:
        """Create an instance of EmbedRequest from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # set to None if provisioned_throughput_id (nullable) is None
        if self.provisioned_throughput_id is None:
            _dict["provisioned_throughput_id"] = None

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> EmbedRequest:
        """Create an instance of EmbedRequest from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return EmbedRequest.model_validate(obj)

        _obj = EmbedRequest.model_validate(
            {
                "model": obj.get("model"),
                "text": obj.get("text"),
                "provisioned_throughput_id": obj.get("provisioned_throughput_id"),
            }
        )

        return _obj


class EmbedRequestModel:
    def __init__(
        self,
        model: str,
        text: list[str],
        # optional properties
        provisioned_throughput_id: Optional[str] = None,
    ):
        """A model object representing the EmbedRequest resource.

                Constructs an object of type EmbedRequest with the provided properties.

                Parameters
                __________
                model : str
                    Identifier of the model to use for generating embeddings.
        Refer to Snowflake documentation for the list of supported models.

        Examples:
          - `snowflake-arctic-embed-m`
          - `snowflake-arctic-embed-m-v1.5`

                text : list[str]
                    An array of input texts for which vector embeddings will be calculated.
        Example: ["Hello world", "Machine learning is fascinating"]

                provisioned_throughput_id : str, optional
                    The provisioned throughput ID to be used with the request.
        """
        self.model = model
        self.text = text
        self.provisioned_throughput_id = provisioned_throughput_id

    __properties = ["model", "text", "provisioned_throughput_id"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return EmbedRequest(
            model=self.model,
            text=self.text,
            provisioned_throughput_id=self.provisioned_throughput_id,
        )

    @classmethod
    def _from_model(cls, model) -> EmbedRequestModel:
        return EmbedRequestModel(
            model=model.model,
            text=model.text,
            provisioned_throughput_id=model.provisioned_throughput_id,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a EmbedRequest.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the EmbedRequest object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> EmbedRequestModel:
        """Create an instance of EmbedRequest from a dict.

        This method constructs a EmbedRequest object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        EmbedRequest
            A EmbedRequest object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(EmbedRequest.from_dict(obj))


EmbedRequest._model_class = EmbedRequestModel



"""
Snowflake External Volume API.

The Snowflake External Volume API is a REST API that you can use to access, update, and perform certain actions on External Volume resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any

from pydantic import ConfigDict, StrictStr

from snowflake.core.external_volume._generated.models.storage_location import StorageLocation


class StorageLocationAzure(StorageLocation):
    """A model object representing the StorageLocationAzure resource.

    Constructs an object of type StorageLocationAzure with the provided properties.

    Parameters
    __________

    name : str

    azure_tenant_id : str
        Specifies the ID for your Office 365 tenant that the allowed and blocked storage accounts belong to.
    storage_base_url : str
        Specifies the base URL for your cloud storage location.
    """

    azure_tenant_id: StrictStr

    storage_base_url: StrictStr

    __properties = ["name", "storage_provider"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> StorageLocationAzure:
        """Create an instance of StorageLocationAzure from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["storage_provider"] = StorageLocation.get_child_model_discriminator_value("StorageLocationAzure")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> StorageLocationAzure:
        """Create an instance of StorageLocationAzure from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return StorageLocationAzure.model_validate(obj)

        _obj = StorageLocationAzure.model_validate(
            {
                "name": obj.get("name"),
                "azure_tenant_id": obj.get("azure_tenant_id"),
                "storage_base_url": obj.get("storage_base_url"),
            }
        )

        return _obj


class StorageLocationAzureModel(StorageLocation):
    def __init__(
        self,
        name: str,
        azure_tenant_id: str,
        storage_base_url: str,
        # optional properties
    ):
        """A model object representing the StorageLocationAzure resource.

        Constructs an object of type StorageLocationAzure with the provided properties.

        Parameters
        __________

        name : str

        azure_tenant_id : str
            Specifies the ID for your Office 365 tenant that the allowed and blocked storage accounts belong to.
        storage_base_url : str
            Specifies the base URL for your cloud storage location.
        """
        super().__init__(
            name=name,
        )
        self.azure_tenant_id = azure_tenant_id
        self.storage_base_url = storage_base_url

    __properties = ["name", "storage_provider"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return StorageLocationAzure(
            name=self.name,
            azure_tenant_id=self.azure_tenant_id,
            storage_base_url=self.storage_base_url,
        )

    @classmethod
    def _from_model(cls, model) -> StorageLocationAzureModel:
        return StorageLocationAzureModel(
            name=model.name,
            azure_tenant_id=model.azure_tenant_id,
            storage_base_url=model.storage_base_url,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a StorageLocationAzure.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the StorageLocationAzure object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> StorageLocationAzureModel:
        """Create an instance of StorageLocationAzure from a dict.

        This method constructs a StorageLocationAzure object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        StorageLocationAzure
            A StorageLocationAzure object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(StorageLocationAzure.from_dict(obj))


StorageLocationAzure._model_class = StorageLocationAzureModel



from collections.abc import Iterator
from typing import TYPE_CHECKING, Optional

from snowflake.core import PollingOperation
from snowflake.core._common import CreateMode
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core._internal.utils import deprecated
from snowflake.core.image_repository._generated.api.image_repository_api_base import (
    ImageRepositoryCollectionBase,
    ImageRepositoryResourceBase,
)


if TYPE_CHECKING:
    from snowflake.core.schema import SchemaResource


from snowflake.core.image_repository._generated.models.image_repository import ImageRepositoryModel as ImageRepository


class ImageRepositoryCollection(ImageRepositoryCollectionBase):
    """Represents the collection operations on the Snowflake Image Repository resource.

    With this collection, you can create, iterate through, and search for image repositories that you have access to
    in the current context.

    Examples
    ________
    Creating an image repository instance:

    >>> image_repository = ImageRepository(name="my_image_repository")
    >>> image_repositories = root.databases["my_db"].schemas["my_schema"].image_repositories
    >>> image_repositories.create(image_repository)
    """

    def __init__(self, schema: "SchemaResource") -> None:
        super().__init__(schema, ImageRepositoryResource)

    @api_telemetry
    def create(  # type: ignore[override]
        self,
        image_repository: ImageRepository,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "ImageRepositoryResource":
        """Create an image repository in Snowflake.

        Parameters
        __________
        image_repository: ImageRepository
            The ``ImageRepository`` object, together with the ``ImageRepository``'s properties:
            name;
        mode: CreateMode, optional
            One of the following enum values.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError` if the image
            repository already exists in Snowflake. Equivalent to SQL ``create image repository <name> ...``.

            ``CreateMode.or_replace``: Replace if the image repository already exists in Snowflake. Equivalent to SQL
            ``create or replace image repository <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the image repository already exists in Snowflake.
            Equivalent to SQL ``create image repository <name> if not exists...``

            Default value is ``CreateMode.error_if_exists``.

        Examples
        ________
        Creating an image repository, replacing an existing image repository with the same name:

        >>> image_repository = ImageRepository(name="my_image_repository")
        >>> image_repositories = root.databases["my_db"].schemas["my_schema"].image_repositories
        >>> image_repositories.create(image_repository, mode=CreateMode.or_replace)
        """
        return super().create(image_repository=image_repository._to_model(), mode=mode)

    @api_telemetry
    def create_async(  # type: ignore[override]
        self,
        image_repository: ImageRepository,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> PollingOperation["ImageRepositoryResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        return super().create_async(image_repository=image_repository._to_model(), mode=mode)

    @api_telemetry
    def iter(self, *, like: Optional[str] = None) -> Iterator[ImageRepository]:  # type: ignore[override]
        """Iterate through ``ImageRepository`` objects from Snowflake, filtering on any optional 'like' pattern.

        Parameters
        __________
        like: str, optional
            A case-insensitive string functioning as a filter, with support for SQL
            wildcard characters (% and _).

        Examples
        ________
        Showing all image repositories that you have access to see:

        >>> image_repositories = image_repository_collection.iter()

        Showing information of the exact image repository you want to see:

        >>> image_repositories = image_repository_collection.iter(like="your-image-repository-name")

        Showing image repositories starting with 'your-image-repository-name-':

        >>> image_repositories = image_repository_collection.iter(like="your-image-repository-name-%")

        Using a for loop to retrieve information from iterator:

        >>> for image_repository in image_repositories:
        >>>     print(image_repository.name)
        """
        image_repositories = super().iter(like=like)
        return map(ImageRepository._from_model, iter(image_repositories))

    @api_telemetry
    def iter_async(self, *, like: Optional[str] = None) -> PollingOperation[Iterator[ImageRepository]]:  # type: ignore[override]
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = super().iter_async(like=like)
        return PollingOperation(future, lambda rest_models: map(ImageRepository._from_model, iter(rest_models)))


class ImageRepositoryResource(ImageRepositoryResourceBase):
    """Represents a reference to a Snowflake image repository.

    With this image repository reference, you can fetch information about an image repository,
    as well as perform certain actions on it.
    """

    _plural_name = "image_repositories"

    @api_telemetry
    def fetch(self) -> ImageRepository:  # type: ignore[override]
        """Fetch the details of an image repository.

        Examples
        ________
        Fetching a reference to an image repository to print its name and properties:

        >>> my_image_repository = image_repository_reference.fetch()
        >>> print(my_image_repository.name)
        """
        return ImageRepository._from_model(super().fetch())

    @api_telemetry
    def fetch_async(self) -> PollingOperation[ImageRepository]:  # type: ignore[override]
        """An asynchronous version of :func:`fetch`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = super().fetch_async()
        return PollingOperation(future, lambda rest_model: ImageRepository._from_model(rest_model))

    @api_telemetry
    @deprecated("drop")
    def delete(self) -> None:
        """Delete this image repository.

        Examples
        ________
        Deleting an image repository using its reference:

        >>> image_repository_reference.delete()
        """
        self.drop()



# coding: utf-8

# flake8: noqa

"""
Snowflake User Defined Function API.

The Snowflake User Defined Function API is a REST API that you can use to access, update, and perform certain actions on User Defined Function resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.user_defined_function._generated.api.user_defined_function_api import UserDefinedFunctionApi

# import models into sdk package
from snowflake.core.user_defined_function._generated.models.argument import Argument
from snowflake.core.user_defined_function._generated.models.base_language import BaseLanguage
from snowflake.core.user_defined_function._generated.models.column_type import ColumnType
from snowflake.core.user_defined_function._generated.models.error_response import ErrorResponse
from snowflake.core.user_defined_function._generated.models.function_language import FunctionLanguage
from snowflake.core.user_defined_function._generated.models.java_function import JavaFunction
from snowflake.core.user_defined_function._generated.models.java_script_function import JavaScriptFunction
from snowflake.core.user_defined_function._generated.models.python_function import PythonFunction
from snowflake.core.user_defined_function._generated.models.return_data_type import ReturnDataType
from snowflake.core.user_defined_function._generated.models.return_table import ReturnTable
from snowflake.core.user_defined_function._generated.models.return_type import ReturnType
from snowflake.core.user_defined_function._generated.models.sql_function import SQLFunction
from snowflake.core.user_defined_function._generated.models.scala_function import ScalaFunction
from snowflake.core.user_defined_function._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.user_defined_function._generated.models.success_response import SuccessResponse
from snowflake.core.user_defined_function._generated.models.tag_assignment import TagAssignment
from snowflake.core.user_defined_function._generated.models.tag_reference import TagReference
from snowflake.core.user_defined_function._generated.models.user_defined_function import UserDefinedFunction
from snowflake.core.user_defined_function._generated.models.user_defined_function_argument import (
    UserDefinedFunctionArgument,
)



# coding: utf-8

# flake8: noqa
"""
Snowflake Notification Integration API.

The Snowflake Notification Integration API is a REST API that you can use to access, update, and perform certain actions on Notification Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

# import models into model package
from snowflake.core.notification_integration._generated.models.error_response import ErrorResponse
from snowflake.core.notification_integration._generated.models.notification_email import NotificationEmail
from snowflake.core.notification_integration._generated.models.notification_hook import NotificationHook
from snowflake.core.notification_integration._generated.models.notification_integration import NotificationIntegration
from snowflake.core.notification_integration._generated.models.notification_queue_aws_sns_outbound import (
    NotificationQueueAwsSnsOutbound,
)
from snowflake.core.notification_integration._generated.models.notification_queue_azure_event_grid_inbound import (
    NotificationQueueAzureEventGridInbound,
)
from snowflake.core.notification_integration._generated.models.notification_queue_azure_event_grid_outbound import (
    NotificationQueueAzureEventGridOutbound,
)
from snowflake.core.notification_integration._generated.models.notification_queue_gcp_pubsub_inbound import (
    NotificationQueueGcpPubsubInbound,
)
from snowflake.core.notification_integration._generated.models.notification_queue_gcp_pubsub_outbound import (
    NotificationQueueGcpPubsubOutbound,
)
from snowflake.core.notification_integration._generated.models.notification_webhook import NotificationWebhook
from snowflake.core.notification_integration._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.notification_integration._generated.models.success_response import SuccessResponse
from snowflake.core.notification_integration._generated.models.tag_assignment import TagAssignment
from snowflake.core.notification_integration._generated.models.tag_reference import TagReference
from snowflake.core.notification_integration._generated.models.webhook_secret import WebhookSecret

__all__ = [
    "ErrorResponse",
    "NotificationEmail",
    "NotificationHook",
    "NotificationIntegration",
    "NotificationQueueAwsSnsOutbound",
    "NotificationQueueAzureEventGridInbound",
    "NotificationQueueAzureEventGridOutbound",
    "NotificationQueueGcpPubsubInbound",
    "NotificationQueueGcpPubsubOutbound",
    "NotificationWebhook",
    "SuccessAcceptedResponse",
    "SuccessResponse",
    "TagAssignment",
    "TagReference",
    "WebhookSecret",
]



from collections.abc import Iterator
from typing import TYPE_CHECKING, Any, Optional

from pydantic import StrictStr

from snowflake.core import PollingOperation
from snowflake.core._common import CreateMode, DatabaseObjectCollectionParent, DatabaseObjectReferenceMixin, DeleteMode
from snowflake.core._generated.api_client import StoredProcApiClient
from snowflake.core._internal.telemetry import api_telemetry
from snowflake.core._operation import PollingOperations
from snowflake.core.database_role._generated.api import DatabaseRoleApi
from snowflake.core.database_role._generated.models import DatabaseRole, DatabaseRoleClone
from snowflake.core.database_role._generated.models.containing_scope import ContainingScope
from snowflake.core.database_role._generated.models.grant import Grant
from snowflake.core.database_role._generated.models.securable import Securable

from .._utils import tag_assignment_to_tag_tuple, tag_resource_to_tag_reference, tag_tuple_to_tag_assignment
from ..tag import TagResource, TagValue
from ._generated import TagAssignment, TagReference


if TYPE_CHECKING:
    from snowflake.core import Root

    from ..database import DatabaseResource


class DatabaseRoleCollection(DatabaseObjectCollectionParent["DatabaseRoleResource"]):
    """Represents the collection operations on the Snowflake DatabaseRole resource.

    With this collection, you can create, iterate through, and search for database roles that you have access to in the
    current context.

    Examples
    ________
    Creating a database role instance:

    >>> database_role_collection = root.databases["my_db"].database_roles
    >>> database_role_collection.create(DatabaseRole(name="test-role", comment="samplecomment"))
    """

    def __init__(self, database: "DatabaseResource", root: "Root") -> None:
        super().__init__(database, DatabaseRoleResource)
        self._api = DatabaseRoleApi(
            root=root, resource_class=self._ref_class, sproc_client=StoredProcApiClient(root=root)
        )

    @api_telemetry
    def create(
        self, database_role: DatabaseRole, *, mode: CreateMode = CreateMode.error_if_exists
    ) -> "DatabaseRoleResource":
        """Create a database role in Snowflake.

        Parameters
        __________
        database_role: DatabaseRole
            The ``DatabaseRole`` object, together with the ``DatabaseRole``'s properties:
            name; comment is optional
        mode: CreateMode, optional
            One of the following enum values.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError`
            if the database role already exists in Snowflake.  Equivalent to SQL ``create database role <name> ...``.

            ``CreateMode.or_replace``: Replace if the database role already exists in Snowflake. Equivalent to SQL
            ``create or replace database role <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the database role already exists in Snowflake.
            Equivalent to SQL ``create database role <name> if not exists...``

            Default is ``CreateMode.error_if_exists``.

        Examples
        ________
        Creating a database role, replacing any existing database role with the same name:

        >>> database_role = DatabaseRole(name="test-role", comment="samplecomment")
        >>> database_role_ref = root.databases["my_db"].database_roles.create(
        ...     database_role, mode=CreateMode.or_replace
        ... )
        """
        real_mode = CreateMode[mode].value
        self._api.create_database_role(self.database.name, database_role, StrictStr(real_mode), async_req=False)
        return self[database_role.name]

    @api_telemetry
    def create_async(
        self, database_role: DatabaseRole, *, mode: CreateMode = CreateMode.error_if_exists
    ) -> PollingOperation["DatabaseRoleResource"]:
        """An asynchronous version of :func:`create`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        real_mode = CreateMode[mode].value
        future = self._api.create_database_role(self.database.name, database_role, StrictStr(real_mode), async_req=True)
        return PollingOperation(future, lambda _: self[database_role.name])

    def iter(self, *, limit: Optional[int] = None, from_name: Optional[str] = None) -> Iterator[DatabaseRole]:
        """Iterate through ``DatabaseRole`` objects from Snowflake, filtering on any optional 'from_name' pattern.

        Parameters
        __________
        show_limit: int, optional
            Limit of the maximum number of rows returned by iter(). The default is ``None``, which behaves equivalently
            to show_limit=10000. This value must be between ``1`` and ``10000``.
        from_name: str, optional
            Fetch rows only following the first row whose object name matches
            the specified string. This is case-sensitive and does not have to be the full name.

        Examples
        ________
        Showing all database roles that you have access to see:

        >>> database_roles = database_role_collection.iter()

        Showing roles from 'your-role-name-':

        >>> database_roles = database_role_collection.iter(from_name="your-role-name-")

        Using a for loop to retrieve information from iterator:

        >>> for database_role in database_roles:
        ...     print(database_role.name, database_role.comment)
        """
        database_roles = self._api.list_database_roles(self.database.name, limit, from_name=from_name, async_req=False)
        return iter(database_roles)

    def iter_async(
        self, *, limit: Optional[int] = None, from_name: Optional[str] = None
    ) -> PollingOperation[Iterator[DatabaseRole]]:
        """An asynchronous version of :func:`iter`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self._api.list_database_roles(self.database.name, limit, from_name=from_name, async_req=True)
        return PollingOperations.iterator(future)


class DatabaseRoleResource(DatabaseObjectReferenceMixin[DatabaseRoleCollection]):
    """Represents a reference to a Snowflake database role.

    With this database role reference, you can delete and clone a database role, as well as
    perform certain actions on it.
    """

    def __init__(self, name: str, collection: DatabaseRoleCollection) -> None:
        self.name = name
        self.collection = collection

    @api_telemetry
    def drop(self, if_exists: Optional[bool] = None) -> None:
        """Drop this database role.

        Parameters
        __________
        if_exists : bool
            Check the existence of this database role before dropping it.
            Default is ``None``, which is equivalent to ``False``.

        Examples
        ________
        Deleting a database role using its reference:

        >>> database_role_reference.drop()

        Deleting a database role using its reference if it exists:

        >>> database_role_reference.drop(if_exists=True)
        """
        self.collection._api.delete_database_role(self.database.name, self.name, if_exists=if_exists, async_req=False)

    @api_telemetry
    def drop_async(self, if_exists: Optional[bool] = None) -> PollingOperation[None]:
        """An asynchronous version of :func:`drop`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.delete_database_role(
            self.database.name, self.name, if_exists=if_exists, async_req=True
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def clone(
        self,
        target_database_role: str,
        target_database: Optional[str] = None,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> "DatabaseRoleResource":
        """Drop this database role.

        Parameters
        __________
        target_database_role: str
            The name of the target database role to clone the database role to.
        target_database: str, optional
            The name of the target database to clone the database role to. If not provided,
            the current database is used.
        mode: CreateMode, optional
            One of the following enum values.

            ``CreateMode.error_if_exists``: Throw an :class:`snowflake.core.exceptions.ConflictError`
            if the database role already exists in Snowflake.  Equivalent to SQL ``create database role <name> ...``.

            ``CreateMode.or_replace``: Replace if the database role already exists in Snowflake. Equivalent to SQL
            ``create or replace database role <name> ...``.

            ``CreateMode.if_not_exists``: Do nothing if the database role already exists in Snowflake.
            Equivalent to SQL ``create database role <name> if not exists...``

            Default is ``CreateMode.error_if_exists``.

        Examples
        ________
        Creating a database role clone using its reference:

        >>> new_database_role_reference = database_role_reference.clone("new-role-name")
        """
        real_mode = CreateMode[mode].value
        target_database_role_clone = DatabaseRoleClone(name=target_database_role)
        self.collection._api.clone_database_role(
            self.database.name, self.name, target_database_role_clone, real_mode, target_database, async_req=False
        )
        if target_database is None:
            target_database = self.database.name
        return self.root.databases[target_database].database_roles[target_database_role]

    @api_telemetry
    def clone_async(
        self,
        target_database_role: str,
        target_database: Optional[str] = None,
        mode: CreateMode = CreateMode.error_if_exists,
    ) -> PollingOperation["DatabaseRoleResource"]:
        """An asynchronous version of :func:`clone`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        real_mode = CreateMode[mode].value
        target_database_role_clone = DatabaseRoleClone(name=target_database_role)
        future = self.collection._api.clone_database_role(
            self.database.name, self.name, target_database_role_clone, real_mode, target_database, async_req=True
        )

        def finalize(_: Any) -> DatabaseRoleResource:
            nonlocal target_database
            if target_database is None:
                target_database = self.database.name
            return self.root.databases[target_database].database_roles[target_database_role]

        return PollingOperation(future, finalize)

    @api_telemetry
    def grant_role(self, role_type: str, role: Securable) -> None:
        """
        Grant a role to this database role.

        Parameters
        __________
        role_type: str
            The type of role which would be granted.
        role: Securable
            The role which would be granted.

        Examples
        ________
        Using a database role reference to grant a role to it:

        >>> database_role_reference.grant("database role", Securable(name="test_role"))
        """
        grant = Grant(securable_type=role_type, securable=role)
        self.collection._api.grant_privileges(self.database.name, self.name, grant, async_req=False)

    @api_telemetry
    def grant_role_async(self, role_type: str, role: Securable) -> PollingOperation[None]:
        """An asynchronous version of :func:`grant_role`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(securable_type=role_type, securable=role)
        future = self.collection._api.grant_privileges(self.database.name, self.name, grant, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def grant_privileges(
        self,
        privileges: list[str],
        securable_type: str,
        securable: Optional[Securable] = None,
        grant_option: Optional[bool] = None,
    ) -> None:
        """
        Grant privileges on a securable to this database role.

        Parameters
        __________
        privileges: list[str]
            The list of privileges to be granted.
        securable_type: str
            The type of securable on which the privileges would be granted.
        securable: Securable, optional
            The securable on which the privileges would be granted. Default is None.
        grant_option: bool, optional
            If True, the grantee can grant the privilege to others. Default is None which means False.

        Examples
        ________
        Using a database role reference to grant privileges to it:

        >>> database_role_reference.grant_privileges(
        ...     ["CREATE", "USAGE"], "database", Securable(database="test_db")
        ... )
        """
        grant = Grant(
            privileges=privileges, securable_type=securable_type, securable=securable, grant_option=grant_option
        )
        self.collection._api.grant_privileges(self.database.name, self.name, grant, async_req=False)

    @api_telemetry
    def grant_privileges_async(
        self,
        privileges: list[str],
        securable_type: str,
        securable: Optional[Securable] = None,
        grant_option: Optional[bool] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`grant_privileges`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(
            privileges=privileges, securable_type=securable_type, securable=securable, grant_option=grant_option
        )
        future = self.collection._api.grant_privileges(self.database.name, self.name, grant, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def grant_privileges_on_all(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        grant_option: Optional[bool] = None,
    ) -> None:
        """
        Grant privileges on all securables matching the criteria to this database role.

        Parameters
        __________
        privileges: list[str]
            The list of privileges to be granted.
        securable_type: str
            The type of securable on which the privileges would be granted.
        containing_scope: ContainingScope
            The criteria to match the securables.
        grant_option: bool, optional
            If True, the grantee can grant the privilege to others. Default is None which means False.

        Examples
        ________
        Using a database role reference to grant privileges on all schemas in a database to it:

        >>> database_role_reference.grant_privileges_on_all(
        ...     ["CREATE", "USAGE"], "schema", ContainingScope(database="test_db")
        ... )
        """
        grant = Grant(
            privileges=privileges,
            securable_type=securable_type,
            containing_scope=containing_scope,
            grant_option=grant_option,
        )
        self.collection._api.grant_privileges(self.database.name, self.name, grant, async_req=False)

    @api_telemetry
    def grant_privileges_on_all_async(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        grant_option: Optional[bool] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`grant_privileges_on_all`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(
            privileges=privileges,
            securable_type=securable_type,
            containing_scope=containing_scope,
            grant_option=grant_option,
        )
        future = self.collection._api.grant_privileges(self.database.name, self.name, grant, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def grant_future_privileges(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        grant_option: Optional[bool] = None,
    ) -> None:
        """
        Grant privileges on all future securables matching the criteria to this database role.

        Parameters
        __________
        privileges: list[str]
            The list of privileges to be granted.
        securable_type: str
            The type of securable on which the privileges would be granted.
        containing_scope: ContainingScope
            The criteria to match the securables.
        grant_option: bool, optional
            If True, the grantee can grant the privilege to others. Default is None which means False.

        Examples
        ________
        Using a database role reference to grant privileges on all future schemas in a database to it:

        >>> database_role_reference.grant_future_privileges(
        ...     ["CREATE", "USAGE"], "schema", ContainingScope(database="test_db")
        ... )
        """
        grant = Grant(
            privileges=privileges,
            securable_type=securable_type,
            containing_scope=containing_scope,
            grant_option=grant_option,
        )
        self.collection._api.grant_future_privileges(self.database.name, self.name, grant, async_req=False)

    @api_telemetry
    def grant_future_privileges_async(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        grant_option: Optional[bool] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`grant_future_privileges`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(
            privileges=privileges,
            securable_type=securable_type,
            containing_scope=containing_scope,
            grant_option=grant_option,
        )
        future = self.collection._api.grant_future_privileges(self.database.name, self.name, grant, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def revoke_role(self, role_type: str, role: Securable) -> None:
        """
        Revoke a role from this database role.

        Parameters
        __________
        role_type: str
            The type of role which would be revoked.
        role: Securable
            The role which would be revoked.

        Examples
        ________
        Using a database role reference to revoke a role from it:

        >>> database_role_reference.revoke("database role", Securable(name="test_role"))
        """
        grant = Grant(securable_type=role_type, securable=role)
        self.collection._api.revoke_grants(self.database.name, self.name, grant, async_req=False)

    @api_telemetry
    def revoke_role_async(self, role_type: str, role: Securable) -> PollingOperation[None]:
        """An asynchronous version of :func:`revoke_role`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(securable_type=role_type, securable=role)
        future = self.collection._api.revoke_grants(self.database.name, self.name, grant, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def revoke_privileges(
        self,
        privileges: list[str],
        securable_type: str,
        securable: Optional[Securable] = None,
        mode: Optional[DeleteMode] = None,
    ) -> None:
        """
        Revoke privileges on a securable from this database role.

        Parameters
        __________
        privileges: list[str]
            The list of privileges to be revoked.
        securable_type: str
            The type of securable on which the privileges would be revoked.
        securable: Securable, optional
            The securable on which the privileges would be revoked. Default is None.
        mode: DeleteMode, optional
            One of the following enum values.

            ``DeleteMode.restrict``: If the privilege being revoked has been re-granted to another role,
            the REVOKE command fails.

            ``DeleteMode.cascade``: If the privilege being revoked has been re-granted, the REVOKE command
            recursively revokes these dependent grants. If the same privilege on an object has been granted
            to the target role by a different grantor (parallel grant), that grant is not affected and the
            target role retains the privilege.

            Default is None which is equivalent to ``DeleteMode.restrict``.

        Examples
        ________
        Using a database role reference to revoke privileges from it:

        >>> database_role_reference.revoke_privileges(
        ...     ["CREATE", "USAGE"], "database", Securable(database="test_db")
        ... )
        """
        grant = Grant(privileges=privileges, securable_type=securable_type, securable=securable)
        self.collection._api.revoke_grants(self.database.name, self.name, grant, mode, async_req=False)

    @api_telemetry
    def revoke_privileges_async(
        self,
        privileges: list[str],
        securable_type: str,
        securable: Optional[Securable] = None,
        mode: Optional[DeleteMode] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`revoke_privileges`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(privileges=privileges, securable_type=securable_type, securable=securable)
        future = self.collection._api.revoke_grants(self.database.name, self.name, grant, mode, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def revoke_privileges_on_all(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        mode: Optional[DeleteMode] = None,
    ) -> None:
        """
        Revoke privileges on all securables matching the criteria from this database role.

        Parameters
        __________
        privileges: list[str]
            The list of privileges to be revoked.
        securable_type: str
            The type of securable on which the privileges would be revoked.
        containing_scope: ContainingScope
            The criteria to match the securables.
        mode: DeleteMode, optional
            One of the following enum values.

            ``DeleteMode.restrict``: If the privilege being revoked has been re-granted to another role,
            the REVOKE command fails.

            ``DeleteMode.cascade``: If the privilege being revoked has been re-granted, the REVOKE command
            recursively revokes these dependent grants. If the same privilege on an object has been granted
            to the target role by a different grantor (parallel grant), that grant is not affected and the
            target role retains the privilege.

            Default is None which is equivalent to ``DeleteMode.restrict``.

        Examples
        ________
        Using a database role reference to revoke privileges on all schemas in a database from it:

        >>> database_role_reference.revoke_privileges_on_all(
        ...     ["CREATE", "USAGE"], "schema", ContainingScope(database="test_db")
        ... )
        """
        grant = Grant(privileges=privileges, securable_type=securable_type, containing_scope=containing_scope)
        self.collection._api.revoke_grants(self.database.name, self.name, grant, mode, async_req=False)

    @api_telemetry
    def revoke_privileges_on_all_async(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        mode: Optional[DeleteMode] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`revoke_privileges_on_all`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(privileges=privileges, securable_type=securable_type, containing_scope=containing_scope)
        future = self.collection._api.revoke_grants(self.database.name, self.name, grant, mode, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def revoke_future_privileges(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        mode: Optional[DeleteMode] = None,
    ) -> None:
        """
        Revoke privileges on all future securables matching the criteria from this database role.

        Parameters
        __________
        privileges: list[str]
            The list of privileges to be revoked.
        securable_type: str
            The type of securable on which the privileges would be revoked.
        containing_scope: ContainingScope
            The criteria to match the securables.
        mode: DeleteMode, optional
            One of the following enum values.

            ``DeleteMode.restrict``: If the privilege being revoked has been re-granted to another role,
            the REVOKE command fails.

            ``DeleteMode.cascade``: If the privilege being revoked has been re-granted, the REVOKE command
            recursively revokes these dependent grants. If the same privilege on an object has been granted
            to the target role by a different grantor (parallel grant), that grant is not affected and the
            target role retains the privilege.

            Default is None which is equivalent to ``DeleteMode.restrict``.

        Examples
        ________
        Using a database role reference to revoke privileges on all future schemas in a database from it:

        >>> database_role_reference.revoke_future_privileges(
        ...     ["CREATE", "USAGE"], "schema", ContainingScope(database="test_db")
        ... )
        """
        grant = Grant(privileges=privileges, securable_type=securable_type, containing_scope=containing_scope)
        self.collection._api.revoke_future_grants(self.database.name, self.name, grant, mode, async_req=False)

    @api_telemetry
    def revoke_future_privileges_async(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        mode: Optional[DeleteMode] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`revoke_future_privileges`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(privileges=privileges, securable_type=securable_type, containing_scope=containing_scope)
        future = self.collection._api.revoke_future_grants(self.database.name, self.name, grant, mode, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def revoke_grant_option_for_privileges(
        self,
        privileges: list[str],
        securable_type: str,
        securable: Optional[Securable] = None,
        mode: Optional[DeleteMode] = None,
    ) -> None:
        """
        Revoke grant option for privileges on a securable from this database role.

        Parameters
        __________
        privileges: list[str]
            The list of privileges to be revoked.
        securable_type: str
            The type of securable on which the privileges would be revoked.
        securable: Securable, optional
            The securable on which the privileges would be revoked. Default is None.
        mode: DeleteMode, optional
            One of the following enum values.

            ``DeleteMode.restrict``: If the privilege being revoked has been re-granted to another role,
            the REVOKE command fails.

            ``DeleteMode.cascade``: If the privilege being revoked has been re-granted, the REVOKE command
            recursively revokes these dependent grants. If the same privilege on an object has been granted
            to the target role by a different grantor (parallel grant), that grant is not affected and the
            target role retains the privilege.

            Default is None which is equivalent to ``DeleteMode.restrict``.

        Examples
        ________
        Using a database role reference to revoke grant option for privileges from it:

        >>> database_role_reference.revoke_grant_option_for_privileges(
        ...     ["CREATE", "USAGE"], "database", Securable(database="test_db")
        ... )
        """
        grant = Grant(privileges=privileges, securable_type=securable_type, securable=securable, grant_option=True)
        self.collection._api.revoke_grants(self.database.name, self.name, grant, mode, async_req=False)

    @api_telemetry
    def revoke_grant_option_for_privileges_async(
        self,
        privileges: list[str],
        securable_type: str,
        securable: Optional[Securable] = None,
        mode: Optional[DeleteMode] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`revoke_grant_option_for_privileges`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(privileges=privileges, securable_type=securable_type, securable=securable, grant_option=True)
        future = self.collection._api.revoke_grants(self.database.name, self.name, grant, mode, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def revoke_grant_option_for_privileges_on_all(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        mode: Optional[DeleteMode] = None,
    ) -> None:
        """
        Revoke grant option for privileges on all securables matching the criteria from this database role.

        Parameters
        __________
        privileges: list[str]
            The list of privileges to be revoked.
        securable_type: str
            The type of securable on which the privileges would be revoked.
        containing_scope: ContainingScope
            The criteria to match the securables.
        mode: DeleteMode, optional
            One of the following enum values.

            ``DeleteMode.restrict``: If the privilege being revoked has been re-granted to another role,
            the REVOKE command fails.

            ``DeleteMode.cascade``: If the privilege being revoked has been re-granted, the REVOKE command
            recursively revokes these dependent grants. If the same privilege on an object has been granted
            to the target role by a different grantor (parallel grant), that grant is not affected and the
            target role retains the privilege.

            Default is None which is equivalent to ``DeleteMode.restrict``.

        Examples
        ________
        Using a database role reference to revoke grant option for privileges on all schemas in a database from it:

        >>> database_role_reference.revoke_grant_option_for_privileges_on_all(
        ...     ["CREATE", "USAGE"], "schema", ContainingScope(database="test_db")
        ... )
        """
        grant = Grant(
            privileges=privileges, securable_type=securable_type, containing_scope=containing_scope, grant_option=True
        )
        self.collection._api.revoke_grants(self.database.name, self.name, grant, mode, async_req=False)

    @api_telemetry
    def revoke_grant_option_for_privileges_on_all_async(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        mode: Optional[DeleteMode] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`revoke_grant_option_for_privileges_on_all`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(
            privileges=privileges, securable_type=securable_type, containing_scope=containing_scope, grant_option=True
        )
        future = self.collection._api.revoke_grants(self.database.name, self.name, grant, mode, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def revoke_grant_option_for_future_privileges(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        mode: Optional[DeleteMode] = None,
    ) -> None:
        """
        Revoke grant option for privileges on all future securables matching the criteria from this database role.

        Parameters
        __________
        privileges: list[str]
            The list of privileges to be revoked.
        securable_type: str
            The type of securable on which the privileges would be revoked.
        containing_scope: ContainingScope
            The criteria to match the securables.
        mode: DeleteMode, optional
            One of the following enum values.

            ``DeleteMode.restrict``: If the privilege being revoked has been re-granted to another role,
            the REVOKE command fails.

            ``DeleteMode.cascade``: If the privilege being revoked has been re-granted, the REVOKE command
            recursively revokes these dependent grants. If the same privilege on an object has been granted
            to the target role by a different grantor (parallel grant), that grant is not affected and the
            target role retains the privilege.

            Default is None which is equivalent to ``DeleteMode.restrict``.

        Examples
        ________
        Using a database role reference to revoke grant option for privileges on all future schemas in a database from it:

        >>> database_role_reference.revoke_grant_option_for_future_privileges(
        ...     ["CREATE", "USAGE"], "schema", ContainingScope(database="test_db")
        ... )
        """  # noqa: E501
        grant = Grant(
            privileges=privileges, securable_type=securable_type, containing_scope=containing_scope, grant_option=True
        )
        self.collection._api.revoke_future_grants(self.database.name, self.name, grant, mode, async_req=False)

    @api_telemetry
    def revoke_grant_option_for_future_privileges_async(
        self,
        privileges: list[str],
        securable_type: str,
        containing_scope: ContainingScope,
        mode: Optional[DeleteMode] = None,
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`revoke_grant_option_for_future_privileges`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        grant = Grant(
            privileges=privileges, securable_type=securable_type, containing_scope=containing_scope, grant_option=True
        )
        future = self.collection._api.revoke_future_grants(self.database.name, self.name, grant, mode, async_req=True)
        return PollingOperations.empty(future)

    @api_telemetry
    def iter_grants_to(self, show_limit: Optional[int] = None) -> Iterator[Grant]:
        """List grants to this database role.

        Lists all privileges and roles granted to the database role.

        Parameters
        __________
        show_limit: int, optional
            Limit of the maximum number of rows returned by iter(). The default is ``None``, which behaves equivalently
            to show_limit=10000. This value must be between ``1`` and ``10000``.

        Examples
        ________
        Using a database role reference to list grants to it:

        >>> database_role_reference.iter_grants_to()
        """
        grants_list = self.collection._api.list_grants(self.database.name, self.name, show_limit, async_req=False)
        return iter(grants_list)

    @api_telemetry
    def iter_grants_to_async(self, show_limit: Optional[int] = None) -> PollingOperation[Iterator[Grant]]:
        """An asynchronous version of :func:`iter_grants_to`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.list_grants(self.database.name, self.name, show_limit, async_req=True)
        return PollingOperations.iterator(future)

    @api_telemetry
    def iter_future_grants_to(self, show_limit: Optional[int] = None) -> Iterator[Grant]:
        """List future grants to this database role.

        Lists all privileges on new (i.e. future) objects of a specified type in a database
        or schema granted to the database role.

        Parameters
        __________
        show_limit: int, optional
            Limit of the maximum number of rows returned by iter(). The default is ``None``, which behaves equivalently
            to show_limit=10000. This value must be between ``1`` and ``10000``.

        Examples
        ________
        Using a database role reference to list future grants to it:

        >>> database_role_reference.iter_future_grants_to()
        """
        grants_list = self.collection._api.list_future_grants(
            self.database.name, self.name, show_limit, async_req=False
        )
        return iter(grants_list)

    @api_telemetry
    def iter_future_grants_to_async(self, show_limit: Optional[int] = None) -> PollingOperation[Iterator[Grant]]:
        """An asynchronous version of :func:`iter_future_grants_to`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.list_future_grants(self.database.name, self.name, show_limit, async_req=True)
        return PollingOperations.iterator(future)

    @api_telemetry
    def set_tags(self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None) -> None:
        """Set tags on a database role.

        Parameters
        __________
        tags: dict[TagResource, TagValue]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.set_tags(
            database=self.database.name,
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
        )

    @api_telemetry
    def set_tags_async(
        self, tags: dict[TagResource, TagValue], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`set_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.set_tags(
            database=self.database.name,
            name=self.name,
            tag_assignment=[
                tag_tuple_to_tag_assignment(TagAssignment, tag_resource, tag_value)
                for [tag_resource, tag_value] in tags.items()
            ],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def unset_tags(self, tag_resources: set[TagResource], if_exists: Optional[bool] = None) -> None:
        """Unset tags from a database role.

        Parameters
        __________
        tag_resources: set[TagResource]
             (required)
        if_exists: bool
             Parameter that specifies how to handle the request for a resource that does not exist: - `true`:
             The endpoint does not throw an error if the resource does not exist. It returns a 200 success response,
             but does not take any action on the resource. - `false`: The endpoint throws an error if the resource
             doesn't exist.
        """
        self.collection._api.unset_tags(
            database=self.database.name,
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
        )

    @api_telemetry
    def unset_tags_async(
        self, tag_resources: set[TagResource], if_exists: Optional[bool] = None
    ) -> PollingOperation[None]:
        """An asynchronous version of :func:`unset_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.unset_tags(
            database=self.database.name,
            name=self.name,
            tag_reference=[tag_resource_to_tag_reference(TagReference, tag_resource) for tag_resource in tag_resources],
            if_exists=if_exists,
            async_req=True,
        )
        return PollingOperations.empty(future)

    @api_telemetry
    def get_tags(self, with_lineage: Optional[bool] = None) -> dict[TagResource, TagValue]:
        """Get the tag assignments for a database role.

        Returns all tags assigned to a database role. This operation requires an active warehouse.

        Parameters
        __________
        with_lineage: bool, optional
            Parameter that specifies whether tag assignments inherited by the object from its ancestors in securable
            object hierarchy should be returned as well: - `true`: All tags assigned to this object should be returned,
            inheritance included. - `false`: Only tags explicitly assigned to this object should be returned.
        """
        tag_assignments = self.collection._api.get_tags(
            database=self.database.name,
            name=self.name,
            with_lineage=with_lineage,
        )
        return dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)

    @api_telemetry
    def get_tags_async(self, with_lineage: Optional[bool] = None) -> PollingOperation[dict[TagResource, TagValue]]:
        """An asynchronous version of :func:`get_tags`.

        Refer to :class:`~snowflake.core.PollingOperation` for more information on asynchronous execution and
        the return type.
        """  # noqa: D401
        future = self.collection._api.get_tags(
            database=self.database.name, name=self.name, with_lineage=with_lineage, async_req=True
        )
        return PollingOperation(
            future, lambda tag_assignments: dict(tag_assignment_to_tag_tuple(ta, self.root) for ta in tag_assignments)
        )
